# -*- coding: utf-8 -*-

from django.contrib.auth.models import AnonymousUser, User
from django.test import RequestFactory, TestCase
from django_stachoutils.middlewares import ExceptionUserInfoMiddleware


class ExceptionUserInfoMiddlewareTest(TestCase):
    rf = RequestFactory()

    @classmethod
    def setUpTestData(cls):
        cls.user = User.objects.create_user('john', 'lennon@thebeatles.com', 'johnpassword')
        cls.anon = AnonymousUser()

    def test_middleware_no_user_request(self):
        request = self.rf.get('/ok/')
        ExceptionUserInfoMiddleware().process_exception(request, BaseException)
        self.assertNotIn('USERNAME', request.META)
        self.assertNotIn('USER_EMAIL', request.META)

    def test_middleware_anonymous_user(self):
        request = self.rf.get('/ok/')
        request.user = self.anon
        ExceptionUserInfoMiddleware().process_exception(request, BaseException)
        self.assertNotIn('USERNAME', request.META)
        self.assertNotIn('USER_EMAIL', request.META)

    def test_middleware_authenticated_user(self):
        request = self.rf.get('/ok/')
        request.user = self.user
        ExceptionUserInfoMiddleware().process_exception(request, BaseException)
        self.assertIn('USERNAME', request.META)
        self.assertIn('USER_EMAIL', request.META)
        self.assertEqual(request.META['USERNAME'], 'john')
        self.assertEqual(request.META['USER_EMAIL'], 'lennon@thebeatles.com')

    def test_middleware_call(self):
        def a_view(request):
            from django.http import HttpResponse
            return HttpResponse("Hello")
        request = self.rf.get('/ok/')
        middleware = ExceptionUserInfoMiddleware(a_view)
        response = middleware(request)
        self.assertEqual(response.content, b'Hello')



from django.contrib import admin
from django.db.models import TextField, CharField
from django_stachoutils.forms.widgets import AdminTexteEditeur, TextInputCounter, TextareaCounter
from my_app import models



class BandMemberAdmin(admin.ModelAdmin):
    formfield_overrides = {
        TextField: {'widget': AdminTexteEditeur(attrs={"class": "vMediumTextField"})},
        CharField: {'widget': TextInputCounter(max_signs=35)},
    }

    class Media:
        js = ("https://code.jquery.com/jquery-2.2.4.min.js",)


class MusicGenreAdmin(admin.ModelAdmin):
    formfield_overrides = {
        TextField: {'widget': TextareaCounter(max_signs=200)},
    }

    class Media:
        js = ("https://code.jquery.com/jquery-2.2.4.min.js",)


admin.site.register(models.BandMember, BandMemberAdmin)
admin.site.register(models.MusicGenre, MusicGenreAdmin)



# -*- coding: utf-8 -*-

import datetime
from django.test import RequestFactory, TestCase
from django_stachoutils import options
from .models import Car
from .utils import set_templates


class PaginationTagTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        Car.objects.create(brand='Saab', name='9.3', price=12500, purchased_on=datetime.date(2015, 7, 29))
        Car.objects.create(brand='Saab', name='900', price=1800, purchased_on=datetime.date(2001, 3, 29))
        Car.objects.create(brand='Saab', name='9.5', price=23700, purchased_on=datetime.date(2012, 2, 11))
        Car.objects.create(brand='Alfa-Romeo', name='Giullia', price=19500, purchased_on=datetime.date(2014, 7, 12))
        Car.objects.create(brand='Alfa-Romeo', name='Sprint', price=11200, purchased_on=datetime.date(2015, 9, 29))
        Car.objects.create(brand='Subaru', name='Forester', price=34500, purchased_on=datetime.date(2016, 1, 29))
        Car.objects.create(brand='Subaru', name='Impreza', price=11200, purchased_on=datetime.date(2012, 6, 29))

    def setUp(self):
        self.queryset = Car.objects.all().order_by('brand', 'name')
        self.rf = RequestFactory()

    @set_templates({'template1': '{% paginator_number page current_page_index get_params %}'})
    def test_paginator_number(self):
        request = self.rf.get('/cars?page=2')
        page = options.paginate(request, self.queryset, paginate_by=3)

        # The usage is to loop over all pages of page (current_page_index being the
        # iterator position).
        result = self.engine.render_to_string('template1', {
            'page': page,
            'current_page_index': 0,
            'get_params': {'page': 2},
        })
        self.assertHTMLEqual(result, '<a href="?page=1">1</a>')

        result = self.engine.render_to_string('template1', {
            'page': page,
            'current_page_index': 1,
            'get_params': {'page': 2},
        })
        self.assertHTMLEqual(result, '<span class="this-page">2</span>')

        result = self.engine.render_to_string('template1', {
            'page': page,
            'current_page_index': 2,
            'get_params': {'page': 2},
        })
        self.assertHTMLEqual(result, '<a class="end" href="?page=3">3</a>')

    @set_templates({'template1': '{% paginate_by_number page paginate_by current_pagination get_params %}'})
    def test_paginate_by_number(self):
        request = self.rf.get('/cars?page=2&paginate_by=3')
        page = options.paginate(request, self.queryset, paginate_by=3)

        result = self.engine.render_to_string('template1', {
            'page': page,
            'paginate_by': 5,
            'get_params': {'page': 2, 'paginate_by': 3},
        })
        self.assertHTMLEqual(result, '<a href="?page=2&paginate_by=5">5</a>')

    @set_templates({'template1': '{% pagination page get_params %}'})
    def test_pagination(self):
        request = self.rf.get('/cars?page=2&paginate_by=3')
        page = options.paginate(request, self.queryset, paginate_by=3)
        result = self.engine.render_to_string('template1', {
            'page': page,
            'get_params': {'page': 2, 'paginate_by': 3},
        })
        self.assertHTMLEqual(result,
            """
            <p class="paginator">
                <a href="?page=1">1</a>
                <span class="this-page">2</span>
                <a href="?page=3" class="end">3</a>
            7 entrées
            </p>
            """
        )

    @set_templates({'template1': '{% pagination page get_params %}'})
    def test_pagination_with_more_than_10_pages(self):
        Car.objects.create(brand='Volvo', name='850', price=11500, purchased_on=datetime.date(2011, 7, 29))
        Car.objects.create(brand='Triumph', name='SpitFire', price=5800, purchased_on=datetime.date(2002, 4, 29))
        Car.objects.create(brand='Fiat', name='126', price=2700, purchased_on=datetime.date(2014, 2, 11))
        Car.objects.create(brand='Honda', name='Civic', price=9500, purchased_on=datetime.date(2012, 7, 11))
        Car.objects.create(brand='Honda', name='S2000', price=11900, purchased_on=datetime.date(2010, 9, 9))
        Car.objects.create(brand='Mercedes', name='Class G', price=44500, purchased_on=datetime.date(2017, 1, 21))
        Car.objects.create(brand='Jaguar', name='XJ12', price=12200, purchased_on=datetime.date(2011, 5, 29))

        request = self.rf.get('/cars?page=10&paginate_by=1')
        page = options.paginate(request, self.queryset, paginate_by=1)
        result = self.engine.render_to_string('template1', {
            'page': page,
            'get_params': {'page': 10, 'paginate_by': 1},
        })
        self.assertHTMLEqual(result,
            """
            <p class="paginator">
                <a href="?page=1">1</a>
                <a href="?page=2">2</a>
                ...
                <a href="?page=8">8</a>
                <a href="?page=9">9</a>
                <span class="this-page">10</span>
                <a href="?page=11">11</a>
                <a href="?page=12">12</a>
                <a href="?page=13">13</a>
                <a href="?page=14" class="end">14</a>
            14 entrées
            </p>
            """
        )

        request = self.rf.get('/cars?page=1&paginate_by=1')
        page = options.paginate(request, self.queryset, paginate_by=1)
        result = self.engine.render_to_string('template1', {
            'page': page,
            'get_params': {'page': 1, 'paginate_by': 1},
        })
        self.assertHTMLEqual(result,
            """
            <p class="paginator">
                <span class="this-page">1</span>
                <a href="?page=2">2</a>
                <a href="?page=3">3</a>
                <a href="?page=4">4</a>
                <a href="?page=5">5</a>
                ...
                <a href="?page=13">13</a>
                <a href="?page=14" class="end">14</a>
            14 entrées
            </p>
            """
        )



# -*- coding: utf-8 -*-

from builtins import range
from builtins import object
from django import forms
from django.forms.models import ModelFormMetaclass
from django.utils.encoding import force_str
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from future.utils import with_metaclass


# http://djangosnippets.org/snippets/2248/

class ModelFormOptions(object):
    def __init__(self, options=None):
        self.inlines = getattr(options, 'inlines', {})


class ModelFormMetaclass(ModelFormMetaclass):
    """ Ajoute les attributs self._forms.inlines """
    def __new__(cls, name, bases, attrs):
        new_class = super(ModelFormMetaclass, cls).__new__(cls, name, bases, attrs)
        new_class._forms = ModelFormOptions(getattr(new_class, 'Forms', None))
        return new_class


class ModelForm(with_metaclass(ModelFormMetaclass, forms.ModelForm)):
    """
    Add to ModelForm the ability to declare inline formsets.

    It save you from the boiler-plate implementation of cross validation/saving of such forms
    in the views.
    You should use It in the admin's forms if you need the inherit them in your apps because
    there is not multi-inherance.

    >>> from myapp import models
    >>> class Program(models.Model):
    ...     name = models.CharField(max_length=100, blank=True)

    >>> class ImageProgram(models.Model):
    ...     image = models.ImageField('image')
    ...     program = models.ForeignKey(Program)

    >>> class Ringtone(models.Model):
    ...     sound = models.FileField('sound')
    ...     program = models.ForeignKey(Program)

    Use It in your admin.py instead of django.forms.ModelForm:
    >>> class ProgramAdminForm(ModelForm):
    ...     class Meta:
    ...         model = Program
    ...         def clean(self):
    ...             cleaned_data = self.cleaned_data
    ...             # stuff
    ...             return cleaned_data

    In your app, say you declare the following inline formsets:
    >>> from django.forms.models import inlineformset_factory
    >>> from myapp.forms import ImageProgramForm, RingtoneProgramForm
    >>> ImageProgramFormSet = inlineformset_factory(models.Program, models.ImageProgram,
    ...                                             form=ImageProgramForm, max_num=6)
    >>> RingToneFormSet = inlineformset_factory(models.Program, models.RingTone,
    ...                                         form=RingtoneProgramForm)

    You can bind them in your program's form:
    >>> class MyProgramForm(ProgramAdminForm):
    ...     class Forms:
    ...         inlines = {
    ...             'images': ImageProgramFormSet,
    ...             'ringtones': RingToneFormSet,
    ...         }

    And instanciate It:
    >>> def my_view(request):
    ...     program_form = MyProgramForm(request.POST, request.FILES, prefix='prog')

    In the template, you access the inlines like that :
    {{ program_form.inlineformsets.images.management_form }}
    {{ program_form.inlineformsets.images.non_form_errors }}
    <table>
    {{ program_form.inlineformsets.images.as_table }}
    </table>


    """

    def __init__(self, *args, **kwargs):
        super(ModelForm, self).__init__(*args, **kwargs)
        if hasattr(self._forms, 'inlines'):
            self.inlineformsets = {}
            fset_initial = kwargs.get('initial', {}).get("_INLINES", {})
            for key, FormSet in self._forms.inlines.items():
                initial = fset_initial.get(key, {})
                self.inlineformsets[key] = FormSet(self.data or None, self.files or None,
                                                   prefix=self._get_formset_prefix(key),
                                                   initial=initial, instance=self.instance)

    def save(self, *args, **kwargs):
        instance = super(ModelForm, self).save(*args, **kwargs)
        if hasattr(self._forms, 'inlines'):
            for fset in self.inlineformsets.values():
                if not fset.has_changed():
                    continue
                fset.save()
        return instance

    def has_changed(self, *args, **kwargs):
        has_changed = super(ModelForm, self).has_changed(*args, **kwargs)
        if has_changed:
            return True
        else:
            for fset in self.inlineformsets.values():
                for i in range(0, fset.total_form_count()):
                    form = fset.forms[i]
                    if form.has_changed():
                        return True
        return False

    def _get_formset_prefix(self, key):
        formset_prefix = key.upper()
        if self.prefix:
            formset_prefix = u'{}_{}'.format(self.prefix, formset_prefix)
        return formset_prefix

    def _clean_form(self):
        super(ModelForm, self)._clean_form()
        for key, fset in self.inlineformsets.items():
            if any(fset.errors):
                self._errors['_%s' % fset.prefix] = fset.non_form_errors()
            for i in range(0, fset.total_form_count()):
                f = fset.forms[i]
                if not f.has_changed():
                    continue
                if f.errors:
                    self._errors['_%s_%d' % (fset.prefix, i)] = f.non_field_errors()

    # This should be in forms.Form.
    def as_tr(self):
        return self._html_output(
            normal_row=u'<td%(html_class_attr)s>%(field)s</td>',
            error_row=u'%s',
            row_ender='<span class="row_end"></span>',
            help_text_html=u'',
            errors_on_separate_row=True
        )

    def labels_as_tr(self):
        out = []
        for name in self.fields.keys():
            bf = self[name]
            if bf.is_hidden:
                continue
            if bf.label:
                label = conditional_escape(force_str(bf.label))
                label = bf.label_tag(label, label_suffix="") or ''
            else:
                label = ''
            out.append(u"<th>%s</th>" % force_str(label))

        return mark_safe(u'<tr>%s</tr>' % ''.join(out))



from django.template.engine import Engine
from functools import wraps


def set_templates(templates):
    templates = dict((name, '{% load stachoutils_extras i18n %}' + html) for name, html in templates.items())
    loaders = [
        ('django.template.loaders.cached.Loader', [
            ('django.template.loaders.locmem.Loader', templates),
        ]),
        'django.template.loaders.app_directories.Loader',
    ]
    engine = Engine(
        libraries={
           'stachoutils_extras': 'django_stachoutils.templatetags.stachoutils_extras',
           'i18n': 'django.templatetags.i18n',
        },
        loaders=loaders,
    )
    def decorator(f):
        @wraps(f)
        def wrapper(self):
            self.engine = engine
            f(self)
        return wrapper
    return decorator


def simple_bool_tag(value):
    return "yes" if value else "Hell No!"





# -*- coding: utf-8 -*-

import os
import hotshot
import hotshot.stats
#import test.pystone
from django.core.management.base import BaseCommand
from optparse import make_option


class Command(BaseCommand):
    args = '<path/to/profiling-file.prof>'
    help = u"Génération de statistiques à partir d'un fichier du profiler Hotspot."

    option_list = BaseCommand.option_list + (
        make_option('-n', '--nb-lines',
                    action='store',
                    type="int",
                    dest='nb_lines',
                    default=50,
                    help=u'Nombre de lignes à afficher dans le rapport.'),
    )

    def handle(self, *args, **options):
        self.stdout.write('')
        try:
            filename = args[0]
        except IndexError:
            self.stderr.write("You must provide the path of a .prof file as argument.")
            return
        else:
            prof_filename = os.path.abspath(filename)

        stats = hotshot.stats.load(prof_filename)
        #stats.strip_dirs()
        stats.sort_stats('time', 'calls')
        stats.print_stats(options.get('nb_lines'))



# -*- coding: utf-8 -*-

from django.db import models


class Car(models.Model):
    name = models.CharField(max_length=100)
    brand = models.CharField(max_length=100)
    price = models.IntegerField()
    purchased_on = models.DateField()
    for_sale = models.BooleanField(default=False)
    last_driver = models.ForeignKey('Person', on_delete=models.CASCADE, null=True, blank=True)
    maintained_by = models.ForeignKey('Garage', on_delete=models.CASCADE, null=True, blank=True)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return "cars/{}/".format(self.pk)

    def price_html(self):
        return u'<em>{}</em>€'.format(self.price)
    price_html.allow_tags = True
    price_html.short_description = 'price'
    price_html.ordering = ['price', 'name']


class Person(models.Model):
    first_name = models.CharField(max_length=100)
    last_name = models.CharField(max_length=100)
    employed_by = models.ForeignKey('Company', on_delete=models.CASCADE, null=True, blank=True)

    def __str__(self):
        return "{} {}".format(self.first_name, self.last_name)


class Company(models.Model):
    name = models.CharField(max_length=100)
    short_name = models.CharField(max_length=16, null=True, blank=True)

    def __str__(self):
        return self.short_name or self.name


class Garage(models.Model):
    name = models.CharField(max_length=100)
    owner = models.ForeignKey('Person', on_delete=models.CASCADE, null=True, blank=True)

    def __str__(self):
        return self.name



# -*- coding: utf-8 -*-

import datetime
from django.test import TestCase
from unittest import skipIf

from .models import Car
from .utils import set_templates


class FilterTest(TestCase):
    @set_templates({'template1': '{{ val|iconify }}'})
    def test_iconify(self):
        result = self.engine.render_to_string('template1', {'val': True})
        self.assertHTMLEqual(result, '<img src="/static/django_stachoutils/img/icon-yes.gif" />')

        result = self.engine.render_to_string('template1', {'val': False})
        self.assertHTMLEqual(result, '<img src="/static/django_stachoutils/img/icon-no.gif" />')

    @set_templates({'template1': '{{ val|processing }}'})
    def test_processing(self):
        result = self.engine.render_to_string('template1', {'val': True})
        self.assertHTMLEqual(result, '<img src="/static/django_stachoutils/img/processing.png" />')

        result = self.engine.render_to_string('template1', {'val': False})
        self.assertHTMLEqual(result, '<img src="/static/django_stachoutils/img/icon-yes.gif" />')

    @set_templates({'template1': '{{ obj|default_if_newrecord:current_index }}'})
    def test_default_if_new_record(self):
        car = Car()
        result = self.engine.render_to_string('template1', {'obj': car, 'current_index': 11})
        self.assertHTMLEqual(result, '11')

        car = Car.objects.create(brand='Saab', name='9-5 2.3T', price=23500,
                                 purchased_on=datetime.date(2018, 4, 1))
        result = self.engine.render_to_string('template1', {'obj': car, 'current_index': 11})
        self.assertHTMLEqual(result, '9-5 2.3T')

    @set_templates({'template1': '{{ val|truncate:10 }}'})
    def test_truncate(self):
        result = self.engine.render_to_string('template1', {'val': 'John'})
        self.assertHTMLEqual(result, 'John')

        result = self.engine.render_to_string('template1', {'val': 'John-the-Baptist'})
        self.assertHTMLEqual(result, 'John-the-B...')

    @set_templates({'template1': '{{ val|truncate:by }}'})
    def test_truncate_with_variable(self):
        result = self.engine.render_to_string('template1', {'val': 'John', 'by': 10})
        self.assertHTMLEqual(result, 'John')

        result = self.engine.render_to_string('template1', {'val': 'John-the-Baptist', 'by': 10})
        self.assertHTMLEqual(result, 'John-the-B...')

    @set_templates({'template1': '{{ val|truncate:by }}'})
    def test_bad_truncate(self):
        result = self.engine.render_to_string('template1', {'val': 'John', 'by': 'ten'})
        self.assertHTMLEqual(result, 'John')

        result = self.engine.render_to_string('template1', {'val': 'John-the-Baptist', 'by': 'ten'})
        self.assertHTMLEqual(result, 'John-the-Baptist')

    @set_templates({'template1': '{{ val|mod:3 }}'})
    def test_mod(self):
        result = self.engine.render_to_string('template1', {'val': 10})
        self.assertHTMLEqual(result, '1')

    @set_templates({'template1': '{{ val|progressbar }}'})
    def test_progressbar(self):
        result = self.engine.render_to_string('template1', {'val': 10})
        self.assertHTMLEqual(result,
            """
            <div class="meter-wrap">
                <div class="meter-value" style="background-color: #009ACD; width: 10%;">
                    <div class="meter-text">
                        10
                    </div>
                </div>
            </div>
            """
        )

    @set_templates({'template1': '{{ val|progressbar }}'})
    def test_progressbar_completed(self):
        result = self.engine.render_to_string('template1', {'val': 100})
        self.assertHTMLEqual(result,
            """
            <div class="meter-wrap">
                <div class="meter-value" style="background-color: #BCEE68; width: 100%;">
                    <div class="meter-text">
                        100
                    </div>
                </div>
            </div>
            """
        )

    @set_templates({'template1': '{{ val|progressbar }}'})
    def test_progressbar_with_tuple(self):
        result = self.engine.render_to_string('template1', {'val': (2.0, 5.0)})
        self.assertHTMLEqual(result,
            """
            <div class="meter-wrap">
                <div class="meter-value" style="background-color: #009ACD; width: 40.0%;">
                    <div class="meter-text">
                        2.0/5.0
                    </div>
                </div>
            </div>
            """
        )

    @set_templates({'template1': '{{ val|progressbar }}'})
    def test_progressbar_with_tuple_and_null_values(self):
        result = self.engine.render_to_string('template1', {'val': (0, 0)})
        self.assertHTMLEqual(result,
            """
            <div class="meter-wrap">
                <div class="meter-value" style="background-color: #009ACD; width: 0%;">
                    <div class="meter-text">
                        0/0
                    </div>
                </div>
            </div>
            """
        )

    @skipIf(True, "Need a fix")
    @set_templates({'template1': '{{ val|progressbar }}'})
    def test_progressbar_with_int(self):
        result = self.engine.render_to_string('template1', {'val': (2, 5)})
        self.assertHTMLEqual(result,
            """
            <div class="meter-wrap">
                <div class="meter-value" style="background-color: #009ACD; width: 40%;">
                    <div class="meter-text">
                        2/5
                    </div>
                </div>
            </div>
            """
        )

    @set_templates({'template1': '{{ val|trend_class }}'})
    def test_trend_class(self):
        result = self.engine.render_to_string('template1', {'val': 4})
        self.assertHTMLEqual(result, 'up')

        result = self.engine.render_to_string('template1', {'val': -2})
        self.assertHTMLEqual(result, 'down')



import logging
from django.test import TestCase
from django_stachoutils.log import BufferingSMTPHandler
try:
    from unittest import mock
except:
    import mock


class BufferingSMTPHandlerTest(TestCase):
    mailhost = "my_smtp"
    fromaddr = "me@my_compagny.com"
    toaddrs = ["chris.mannix@redrock.org", "bob.ruth@bassocied-bhunters.com"]
    subject = "Concerning Daisy D."
    capacity = 10

    @mock.patch('smtplib.SMTP', autospec=True)
    def test_buffering_smtp_handler_flush(self, smtp_m):
        logger = logging.getLogger('test')
        logger.setLevel(logging.INFO)
        handler = BufferingSMTPHandler(self.mailhost, self.fromaddr, self.toaddrs,
                                       self.subject, self.capacity, fmt="%(levelname)-5s %(message)s")
        logger.addHandler(handler)

        logger.info(u"I am pleased to announce you that your bounty is ready to be collected.")
        handler.flush()
        smtp_m.assert_called_with(self.mailhost, 25)
        smtp_m().sendmail.assert_called_with(self.fromaddr, self.toaddrs,
            b'From: me@my_compagny.com\r\nTo: chris.mannix@redrock.org,bob.ruth@bassocied-bhunters.com\r\nSubject: Concerning Daisy D.\r\n\r\nINFO  I am pleased to announce you that your bounty is ready to be collected.\r\n')
        smtp_m().quit.assert_called_with()




# -*- coding: utf-8 -*-

import os
import shutil
import sys
import tempfile

from django import forms
from django.conf import settings
from django.core.files.images import ImageFile
from django.test import TestCase, modify_settings

from .models import House, Person


@modify_settings(INSTALLED_APPS={'append': 'django_thumbor', 'remove': 'sorl.thumbnail'})
class ThumborFieldsTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.h1 = House.objects.create(address="12, rue de la Paix", city="Paris")

    def setUp(self):
        self.tmp_media_root = tempfile.mkdtemp()
        # widgets.py has to be reloaded for contextual import to be performed.
        if 'django_stachoutils.forms.widgets' in sys.modules:
            self.widgets_mod = sys.modules.pop('django_stachoutils.forms.widgets')
            self.fields_mod = sys.modules.pop('django_stachoutils.forms.fields')
            self.forms_mod = sys.modules.pop('django_stachoutils.forms')

    def tearDown(self):
        shutil.rmtree(self.tmp_media_root)
        sys.modules['django_stachoutils.forms.widgets'] = self.widgets_mod
        sys.modules['django_stachoutils.forms.fields'] = self.fields_mod
        sys.modules['django_stachoutils.forms'] = self.forms_mod

    def test_bound_image_model_choice_field_with_thumbor(self):
        from django_stachoutils.forms import ImageModelChoiceField

        class PersonForm(forms.ModelForm):
            house = ImageModelChoiceField(related_fieldname="photo",
                                          queryset=House.objects.all())

            class Meta:
                model = Person
                fields = ('name', 'lastname', 'gender', 'house')

        old_media_root = settings.MEDIA_ROOT
        with self.settings(MEDIA_ROOT=self.tmp_media_root):
            with open(os.path.join(old_media_root, "home_1.jpg"), mode="rb") as f:
                self.h1.photo.save("some pic", ImageFile(f))
            form = PersonForm({'name': 'Stan',
                               'lastname': 'Guerra',
                               'gender': 'male',
                               'house': self.h1.pk})
            self.assertHTMLEqual(
                form.as_table(),
                """
                <tr>
                  <th>
                    <label for="id_name">Name:</label>
                  </th>
                  <td>
                    <input type="text" name="name" value="Stan" required id="id_name" maxlength="100" />
                  </td>
                </tr>
                <tr>
                  <th><label for="id_lastname">Lastname:</label></th>
                  <td><input type="text" name="lastname" value="Guerra" required id="id_lastname" maxlength="100" /></td>
                </tr>
                <tr>
                  <th><label for="id_gender">Gender:</label></th>
                  <td>
                    <select name="gender" required id="id_gender">
                      <option value="">---------</option>
                      <option value="male" selected>Male</option>
                      <option value="female">Female</option>
                    </select>
                    <div class="droppableHiddenInput">
                      <input type="hidden" name="house" value="1" />
                      <div class="droppableContainer">
                        <span class="delete" title="Vider l'emplacement"></span>
                        <div class="droppable">
                          <div class="draggable">
                            <img src="http://my_thumbor.com:8888/dj6AHY7DZM35q5rrDngbOZhKp68=/120x0/http%3A//media.my_project.com/houses/12_rue_de_la_Paix_Paris.jpg" width="120">
                          </div>
                        </div>
                      </div>
                      <div class="message"></div>
                    </div>
                  </td>
                </tr>
                """.format()
            )
            self.assertTrue(form.is_valid())
            form.save()

            person = Person.objects.latest('pk')
            self.assertEqual(person.name, 'Stan')
            self.assertEqual(person.house, self.h1)



# -*- coding: utf-8 -*-

from builtins import object
from django import forms
from django.conf import settings
from django.contrib.admin import widgets as admin_widgets
from django.utils.http import quote
from django.utils.safestring import mark_safe

if "django_thumbor" in settings.INSTALLED_APPS:
    from django_thumbor import generate_url
else:
    from django.template import Context, Template


STATICS = {
    'js': [
        'django_stachoutils/js/jquery.charcounter.js',
        'django_stachoutils/js/jquery.fieldselection.js',
        'django_stachoutils/js/editeur.js',
    ]
}


class TexteEditeur(forms.Textarea):
    template_name = 'django_stachoutils/forms/widgets/texte_editeur.html'

    def _media(self):
        return forms.Media(**STATICS)
    media = property(_media)


class AdminTexteEditeur(admin_widgets.AdminTextareaWidget, TexteEditeur):
    pass


class TextareaCounter(AdminTexteEditeur):
    def __init__(self, max_signs=None, direction='up', attrs=None):
        self.max_signs = max_signs
        self.direction = direction
        super(TextareaCounter, self).__init__(attrs)

    def render(self, name, value, attrs=None, renderer=None):
        textarea = super(TextareaCounter, self).render(name, value, attrs)
        jscounter = ''
        if self.max_signs:
            label = "signe(s)"
            if self.direction == 'down':
                label += ' restant(s)'
            jscounter = (
                '<script type="text/javascript">'
                    '$(document).ready(function () { '
                        '$("#%(id)s").charCounter(%(max_signs)d, {'
                              'container: "<div></div>",'
                              'direction: "%(direction)s",'
                              'format: "%%1 %(label)s",'
                              'limit: false,'
                              'limit_class: "error"'
                        '});'
                    '});'
                '</script>' % {
                    'id': attrs['id'],
                    'direction': self.direction,
                    'label': label,
                    'max_signs': self.max_signs})
        return mark_safe(textarea + jscounter)


class TextInputCounter(forms.TextInput):
    def __init__(self, max_signs=None, direction='up', attrs=None):
        self.max_signs = max_signs
        self.direction = direction
        super(TextInputCounter, self).__init__(attrs)

    def render(self, name, value, attrs=None, renderer=None):
        textinput = super(TextInputCounter, self).render(name, value, attrs)
        jscounter = ''
        if self.max_signs:
            label = "signe(s)"
            if self.direction == 'down':
                label += ' restant(s)'
            jscounter = (
                '<script type="text/javascript">'
                    '$(document).ready(function () { '
                        '$("#%(id)s").charCounter(%(max_signs)d, {'
                              'container: "<span></span>",'
                              'direction: "%(direction)s",'
                              'format: "%%1 %(label)s",'
                              'limit: false,'
                              'limit_class: "error"'
                        '});'
                    '});'
                '</script>' % {
                    'id': attrs['id'],
                    'direction': self.direction,
                    'label': label,
                    'max_signs': self.max_signs})
        return mark_safe(textinput + jscounter)

    def _media(self):
        return forms.Media(**STATICS)
    media = property(_media)


class ImageDroppableHiddenInput(forms.HiddenInput):
    def __init__(self, *args, **kwargs):
        super(ImageDroppableHiddenInput, self).__init__(*args, **kwargs)
        self.related_model, self.related_fieldname = None, None
        self.image_container_html = ''
        self.message = ''

    class Media(object):
        css = {
            'all': ('django_stachoutils/css/forms.css',)
        }
        js = ('django_stachoutils/js/forms.js',)

    def render(self, name, value, attrs=None, renderer=None):
        hidden_input = super(ImageDroppableHiddenInput, self).render(name, value, attrs=None)
        image_tag = '<img />'
        if value:
            rel_obj = self.related_model.objects.get(pk=value)  # TODO: Faire un get_object_or_none
            image_tag = self._get_thumbnail(rel_obj)

        tag = (
            '<div class="droppableHiddenInput">%s'
            '    <div class="droppableContainer"><span class="delete" title="Vider l\'emplacement"></span>%s'
            '        <div class="droppable"><div class="draggable">%s</div></div>'
            '    </div>'
            '    <div class="message">%s</div>'
            '</div>' % (hidden_input, self.image_container_html, image_tag, self.message)
        )
        return mark_safe(tag)

    def _get_thumbnail(self, rel_obj):
        image_field = getattr(rel_obj, self.related_fieldname)
        if "django_thumbor" in settings.INSTALLED_APPS:
            return get_thumbor_thumbnail_tag(image_field)
        else:
            t = Template('{% load thumbnail %}{% thumbnail img_field "120" as im %}<img src="{{ im.url }}"'
                         'width="{{ im.width }}" height="{{ im.height }}">{% endthumbnail %}')
            d = {"img_field": image_field}
            return mark_safe(t.render(Context(d)))


def get_thumbor_thumbnail_tag(image, width=120):
    return mark_safe('<img src="%s" width="%d">' % (
        get_thumbor_thumbnail_url(image, width=width),
        width))


def get_thumbor_thumbnail_url(image, **kwargs):
    storage = image.storage
    thumbor_server = settings.THUMBOR_SERVER_EXTERNAL
    url = quote(image.url)
    if hasattr(storage, "key"):
        try:
            url = storage.key(image.name)
        except NotImplementedError:
            pass
        else:
            thumbor_server = settings.THUMBOR_SERVER
    return generate_url(url, thumbor_server=thumbor_server, **kwargs)



# -*- coding: utf-8 -*-

from django import forms
from .widgets import ImageDroppableHiddenInput


class ImageModelChoiceField(forms.ModelChoiceField):
    def __init__(self, related_fieldname, queryset, *args, **kwargs):
        if 'widget' not in kwargs:
            kwargs['widget'] = ImageDroppableHiddenInput
        super(ImageModelChoiceField, self).__init__(queryset, *args, **kwargs)
        self.widget.related_model = queryset.model
        self.widget.related_fieldname = related_fieldname

    def set_image_container_html(self, html):
        self.widget.image_container_html = html

    def set_message(self, html):
        self.widget.message = html



# -*- coding: utf-8 -*-

from django.db import models


class Car(models.Model):
    name = models.CharField(max_length=100)
    brand = models.CharField(max_length=100)
    price = models.IntegerField()
    purchased_on = models.DateField()
    for_sale = models.BooleanField(default=False)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return "cars/{}/".format(self.pk)

    def price_html(self):
        return u'<em>{}</em>€'.format(self.price)
    price_html.allow_tags = True
    price.short_description = 'price'



import os
import time
import tempfile
import hotshot
from django.conf import settings

try:
    PROFILE_LOG_BASE = settings.PROFILE_LOG_BASE
except:
    PROFILE_LOG_BASE = tempfile.gettempdir()


def profile(log_file):
    """Profile some callable.

    This decorator uses the hotshot profiler to profile some callable (like
    a view function or method) and dumps the profile data somewhere sensible
    for later processing and examination.

    It takes one argument, the profile log name. If it's a relative path, it
    places it under the PROFILE_LOG_BASE. It also inserts a time stamp into the
    file name, such that 'my_view.prof' become 'my_view-20100211T170321.prof',
    where the time stamp is in UTC. This makes it easy to run and compare
    multiple trials.
    """

    if not os.path.isabs(log_file):
        log_file = os.path.join(PROFILE_LOG_BASE, log_file)

    def _outer(f):
        def _inner(*args, **kwargs):
            # Add a timestamp to the profile output when the callable
            # is actually called.
            (base, ext) = os.path.splitext(log_file)
            base = base + "-" + time.strftime("%Y%m%dT%H%M%S", time.gmtime())
            final_log_file = base + ext

            prof = hotshot.Profile(final_log_file)
            try:
                ret = prof.runcall(f, *args, **kwargs)
            finally:
                prof.close()
            return ret

        return _inner
    return _outer



from django.db import models
from django_stachoutils.storage import OverwriteStorage


def car_photo_path(instance, filename):
    return 'cars/{}_{}.jpg'.format(instance.brand, instance.name)


class Car(models.Model):
    name = models.CharField(max_length=100)
    brand = models.CharField(max_length=100)
    photo = models.ImageField(upload_to=car_photo_path, storage=OverwriteStorage(), null=True)

    def __str__(self):
        return self.name



# -*- coding: utf-8 -*-

import datetime
from django.test import TestCase
from django.utils.http import urlencode

from .models import Car
from .utils import set_templates, simple_bool_tag


class TableTagTest(TestCase):
    columns = [
        {'label': 'Name', 'field': 'name', 'link': {'href': 'get_absolute_url', 'style': 'nowrap', 'title': '__str__'}},
        {'label': 'Brand', 'field': 'brand'},
        {'label': 'Price', 'field': 'price_html', 'editable': True},
        {'label': 'Purchased Year', 'field': 'purchased_on', 'filters': [('date', 'Y')]},
        {'label': 'For Sale', 'field': 'for_sale', 'tags': [simple_bool_tag]},
    ]

    @set_templates({'template1': '{% table_header_tag columns full_path %}'})
    def test_table_header_tag(self):
        result = self.engine.render_to_string('template1', {
            'columns': self.columns,
            'full_path': "/cars?theme=dark&paginate_by=25"})

        self.assertHTMLEqual(result, """
            <th class="action-checkbox-column"> <input id="action-toggle" type="checkbox" style="display: inline;"></th>
            <th><a href="?%s">Name</a><input class="fieldname" type="hidden" value="name"/></th>
            <th><a href="?%s">Brand</a><input class="fieldname" type="hidden" value="brand"/></th>
            <th><a href="?%s">Price</a><input class="fieldname" type="hidden" value="price_html"/></th>
            <th><a href="?%s">Purchased Year</a><input class="fieldname" type="hidden" value="purchased_on" /></th>
            <th><a href="?%s">For Sale</a><input class="fieldname" type="hidden" value="for_sale" /></th>
        """ % (
            urlencode([('o', '0'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '1'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '2'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '3'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '4'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
        ))

    @set_templates({'template1': '{% table_header_tag columns full_path %}'})
    def test_table_header_tag_sorted_asc(self):
        result = self.engine.render_to_string('template1', {
            'columns': self.columns,
            'full_path': "/cars?theme=dark&paginate_by=25&ot=asc&o=1"})

        self.assertHTMLEqual(result, """
            <th class="action-checkbox-column"> <input id="action-toggle" type="checkbox" style="display: inline;"></th>
            <th><a href="?%s">Name</a><input class="fieldname" type="hidden" value="name"/></th>
            <th class="ascending sorted"><a href="?%s">Brand</a><input class="fieldname" type="hidden" value="brand"/></th>
            <th><a href="?%s">Price</a><input class="fieldname" type="hidden" value="price_html"/></th>
            <th><a href="?%s">Purchased Year</a><input class="fieldname" type="hidden" value="purchased_on" /></th>
            <th><a href="?%s">For Sale</a><input class="fieldname" type="hidden" value="for_sale" /></th>
        """ % (
            urlencode([('o', '0'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '1'), ('ot', 'desc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '2'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '3'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '4'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
        ))

    @set_templates({'template1': '{% table_header_tag columns full_path %}'})
    def test_table_header_tag_sorted_desc(self):
        result = self.engine.render_to_string('template1', {
            'columns': self.columns,
            'full_path': "/cars?theme=dark&paginate_by=25&ot=desc&o=1"})

        self.assertHTMLEqual(result, """
            <th class="action-checkbox-column"> <input id="action-toggle" type="checkbox" style="display: inline;"></th>
            <th><a href="?%s">Name</a><input class="fieldname" type="hidden" value="name"/></th>
            <th class="descending sorted"><a href="?%s">Brand</a><input class="fieldname" type="hidden" value="brand"/></th>
            <th><a href="?%s">Price</a><input class="fieldname" type="hidden" value="price_html"/></th>
            <th><a href="?%s">Purchased Year</a><input class="fieldname" type="hidden" value="purchased_on" /></th>
            <th><a href="?%s">For Sale</a><input class="fieldname" type="hidden" value="for_sale" /></th>
        """ % (
            urlencode([('o', '0'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '1'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '2'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '3'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
            urlencode([('o', '4'), ('ot', 'asc'), ('paginate_by', '25'), ('theme', 'dark')]),
        ))

    @set_templates({'template1': '{% table_row_tag columns car %}'})
    def test_table_row_tag(self):
        car = Car.objects.create(brand='Saab', name='9.3 2.0t', price=12500,
                                 purchased_on=datetime.date(2015, 7, 29),
                                 for_sale=True)
        result = self.engine.render_to_string('template1', {
            'columns': self.columns,
            'car': car})
        self.assertHTMLEqual(result, """
            <td class="pk"><input type="checkbox" name="_selected_action" value="1" class="action-select"></td>
            <td class=""><a class="nowrap" href="cars/1/" title="9.3 2.0t">9.3 2.0t</a></td>
            <td class="">Saab</td>
            <td class="editable"><em>12500</em>€</td>
            <td class="">2015</td>
            <td class="">yes</td>
        """)


class InlineTagTest(TestCase):
    @set_templates({'template1': '{% inline_display_class inlines_visibles inline counter %}'})
    def test_inline_display_class(self):
        result = self.engine.render_to_string('template1', {
            'inlines_visibles': {'caroption_set': '1,3',
                                 'cardriver_set': '0'},
            'inline': 'caroption_set',
            'counter': '3'})
        self.assertHTMLEqual(result, '')

        result = self.engine.render_to_string('template1', {
            'inlines_visibles': {'caroption_set': '1,3',
                                 'cardriver_set': '0'},
            'inline': 'caroption_set',
            'counter': '2'})
        self.assertHTMLEqual(result, 'closed')

        result = self.engine.render_to_string('template1', {
            'inlines_visibles': {'caroption_set': '1,3',
                                 'cardriver_set': '0'},
            'inline': 'carbuyer_set',
            'counter': '2'})
        self.assertHTMLEqual(result, '')

    @set_templates({'template1': '{% inline_display_style inlines_visibles inline counter %}'})
    def test_inline_display_style(self):
        result = self.engine.render_to_string('template1', {
            'inlines_visibles': {'caroption_set': '1,3',
                                 'cardriver_set': '0'},
            'inline': 'caroption_set',
            'counter': '3'})
        self.assertHTMLEqual(result, '')

        result = self.engine.render_to_string('template1', {
            'inlines_visibles': {'caroption_set': '1,3',
                                 'cardriver_set': '0'},
            'inline': 'caroption_set',
            'counter': '2'})
        self.assertHTMLEqual(result, 'display: none;')



# -*- coding: utf-8 -*-

from past.builtins import basestring

from django.conf import settings
from django.template import Library, Context, Template
from django.utils.safestring import mark_safe
from django.utils.http import urlencode
from django.utils.translation import gettext as _

from django_stachoutils import urldecode
from django_stachoutils.views.generic import ORDER_BY_ATTR, ORDER_TYPE_ATTR


DOT = '.'
# TODO: mettre ca dans les settings.
PAGE_VAR = 'page'
PAGINATE_BY_VAR = 'paginate_by'
register = Library()


def paginator_number(page, i, get_params={}):
    if i == DOT:
        return u'... '
    elif i == page.number - 1:
        return mark_safe(u'<span class="this-page">%d</span> ' % (i + 1))
    else:
        params = get_params.copy()
        params.update({PAGE_VAR: i + 1})
        return mark_safe(u'<a href="?%s"%s>%d</a> ' % (urlencode(params), (i == page.paginator.num_pages - 1 and ' class="end"' or ''), i + 1))
paginator_number = register.simple_tag(paginator_number)


def paginate_by_number(page, i, current_pagination, get_params={}):
    if i == DOT:
        return u'... '
    elif i == current_pagination:
        return mark_safe(u'<span class="this-page">%s</span> ' % str(i))
    else:
        params = get_params.copy()
        params.update({PAGINATE_BY_VAR: i})
        params = sorted([(k, v) for k, v in params.items()])
        return mark_safe(u'<a href="?%s"%s>%s</a> ' % (urlencode(params), '', str(i)))
paginate_by_number = register.simple_tag(paginate_by_number)


# Librement inspiré de : http://code.djangoproject.com/browser/django/trunk/django/contrib/admin/templatetags/admin_list.py
def pagination(page, current_pagination, paginates_by=[], get_params={}):
    paginates_by = paginates_by and (paginates_by + ['All']) or []
    paginator = page.paginator
    pagination_required = (paginator.num_pages - 1)
    if not pagination_required:
        page_range = []
    else:
        ON_EACH_SIDE = 3
        ON_ENDS = 2

        if paginator.num_pages <= 10:
            page_range = range(paginator.num_pages)
        else:
            page_range = []
            if page.number > (ON_EACH_SIDE + ON_ENDS):
                page_range.extend(range(0, ON_EACH_SIDE - 1))
                page_range.append(DOT)
                page_range.extend(range(page.number - ON_EACH_SIDE, page.number + 1))
            else:
                page_range.extend(range(0, page.number + 1))
            if page.number < (paginator.num_pages - ON_EACH_SIDE - ON_ENDS - 1):
                page_range.extend(range(page.number + 1, page.number + ON_EACH_SIDE + 1))
                page_range.append(DOT)
                page_range.extend(range(paginator.num_pages - ON_ENDS, paginator.num_pages))
            else:
                page_range.extend(range(page.number + 1, paginator.num_pages))

    return {
        'page': page,
        'pagination_required': pagination_required,
        'paginates_by_required': paginates_by and True or False,
        'page_range': page_range,
        'nb_resultats': page.paginator.count,
        'paginates_by': paginates_by,
        'get_params': get_params,
        'current_pagination': current_pagination,
    }
pagination = register.inclusion_tag('django_stachoutils/pagination.html')(pagination)


@register.filter
def iconify(value):
    """Transforme un boolean en image."""
    return mark_safe("<img src='%sdjango_stachoutils/img/icon-%s.gif' />" % (
        settings.STATIC_URL, value and 'yes' or 'no'))


@register.filter
def processing(value):
    """Show a kind of alert if True."""
    return mark_safe("<img src='%sdjango_stachoutils/img/%s' />" % (
        settings.STATIC_URL, value and 'processing.png' or 'icon-yes.gif'))


@register.filter
def default_if_newrecord(record, default):
    if record.pk:
        return record
    else:
        return default


@register.filter
def truncate(value, arg):
    """ http://djangosnippets.org/snippets/163/ """
    try:
        length = int(arg)
    except ValueError:
        return value
    if not isinstance(value, basestring):
        value = str(value)
    if (len(value) > length):
        return value[:length] + "..."
    else:
        return value


@register.filter
def mod(value, arg):
    return value % arg


@register.filter
def progressbar(value):
    if isinstance(value, tuple):
        progress = value[0]
        total = value[1]
        perc_completed = 0
        label = "%s/%s" % (progress, total)
        if total > 0:
            perc_completed = progress / total * 100  # FIXME with Integers.

    else:
        perc_completed = int(value)
        label = perc_completed

    color = '#009ACD'
    if perc_completed == 100:
        color = '#BCEE68'

    return mark_safe("""
        <div class="meter-wrap">
            <div class="meter-value" style="background-color: %s; width: %s%%;">
                <div class="meter-text">
                    %s
                </div>
            </div>
        </div>""" % (color, perc_completed, label))


DOWN_STYLE = 'down'
UP_STYLE = 'up'


@register.filter
def trend_class(value):
    if value < 0:
        return DOWN_STYLE
    elif value > 0:
        return UP_STYLE


@register.filter
def current_filters(filters):
    """HTML render of the active filters from a generic_list. """
    # Selected filter, except 'All'.
    out = [(f[0], [choice[1] for choice in f[1] if choice[2]]) for f in filters]
    return [mark_safe(u'%s: <strong>%s</strong>' % (k, v[0]))
            for k, v in out if (v and v[0] != _("All"))]


# Générateur des lignes de tableaux d'index génériques.


@register.simple_tag
def table_header_tag(columns, current_url):
    out = ['<th class="action-checkbox-column"> <input id="action-toggle" type="checkbox" style="display: inline;"></th>']
    get_params = urldecode(current_url)
    out += _table_header(columns, get_params)
    return mark_safe(u' '.join(out))


def _table_header(columns, get_params, padding=0, sortable=True):
    out = []
    for i, c in enumerate(columns):
        _get_params = get_params.copy()
        if 'with' in c:
            out += _table_header(c['columns'], _get_params, i, False)
        if 'label' in c:
            style = []
            link = u'%s' % c['label']
            if sortable:
                order_type = 'asc'
                order_by = get_params.get(ORDER_BY_ATTR)
                if order_by and order_by == str(i + padding):
                    style.append('sorted')
                    if get_params[ORDER_TYPE_ATTR] == 'asc':
                        style.append('ascending')
                        order_type = 'desc'
                    else:
                        style.append('descending')
                _get_params.update({ORDER_BY_ATTR: i + padding, ORDER_TYPE_ATTR: order_type})
                _get_params = sorted([(k, v) for k, v in _get_params.items()])
                link = u'<a href="?%s">%s</a>' % (urlencode(_get_params), link)
            out.append(u'<th%s>%s<input class="fieldname" type="hidden" value="%s"/></th>' % (
                (style and ' class="%s"' % ' '.join(style) or ''), link, c['field']
            ))
    return out


@register.simple_tag
def table_row_tag(columns, instance):
    out = ['<td class="pk"><input type="checkbox" name="_selected_action" value="%d" class="action-select"></td>' % instance.pk]
    out += _table_row(columns, instance)
    return mark_safe(u' '.join(out))


def _table_row(columns, instance):
    out = []
    for c in columns:
        if 'with' in c:
            out += _table_row(c['columns'], getattr(instance, c['with'])())
        else:
            style = []
            if 'label' in c:
                value = instance
                # related lookup. On utilise pas qset.values() en amont qui eviterait ce truc récursif
                # car empêche d'accéder des attr virtuels sur les instances.
                for attr in c['field'].split('__'):
                    if value:
                        if hasattr(value, 'items'):
                            value = value.get(attr)
                        else:
                            value = getattr(value, attr)
                            if callable(value):
                                value = value()
                    else:
                        break
                if c.get('render'):
                    params = get_render_params(instance, c.get('render_params'))
                    value = getattr(value, c['render'])(*params)
                if c.get('tags'):
                    for tag in c['tags']:
                        value = tag(value)
                if value and 'link' in c:
                    href, title, a_style = "", "", ""
                    if 'href' in c['link']:
                        href = getattr(instance, c['link']['href'])()
                    if 'title' in c['link']:
                        title = getattr(instance, c['link']['title'])()
                    if 'style' in c['link']:
                        a_style = ' class="%s"' % c["link"]["style"]
                    value = u'<a%s title="%s" href="%s">%s</a>' % (a_style, title, href, value)
                if c.get('editable'):
                    style.append('editable')
                t = (value and value or '-')
                if c.get('filters') and t != '-':
                    t = Template(u'{{ value%s }}' % get_template_filters(c['filters'])).render(Context({'value': t}))
                out.append(u'<td class="%s">%s</td>' % (u' '.join(style), t))
    return out


def get_render_params(instance, params):
    out = []
    if params:
        out = list(params)
        try:
            out[out.index('self')] = instance
        except ValueError:
            pass
    return out


def get_template_filters(filters):
    t = []
    for f in filters:
        if len(f) < 2:
            t.append(u'|%s' % f[0])
        else:
            t.append(u'|%s:"%s"' % (f[0], f[1]))
    return u''.join(t)


## Inline-extras ##


def inline_visible(inlines_visibles, inline_name, current_instance_number):
    visible = True
    current_instance_number = str(current_instance_number)
    if inline_name in inlines_visibles:
        visible = False
        visibles = inlines_visibles[inline_name].split(",")
        if current_instance_number in visibles:
            visible = True
    return visible


@register.simple_tag
def inline_display_class(inlines_visibles, inline_name, current_instance_number):
    out = ""
    if not inline_visible(inlines_visibles, inline_name, current_instance_number):
        out = "closed"
    return mark_safe(out)


@register.simple_tag
def inline_display_style(inlines_visibles, inline_name, current_instance_number):
    out = ""
    if not inline_visible(inlines_visibles, inline_name, current_instance_number):
        out = "display: none;"
    return mark_safe(out)



# -*- coding: utf-8 -*-

import os
import shutil
import tempfile

from django.conf import settings
from django.core.files.images import ImageFile
from django.test import TestCase

from .models import Car


class StorageTest(TestCase):
    def setUp(self):
        self.tmp_media_root = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmp_media_root)

    def test_overwite_storage(self):
        old_media_root = settings.MEDIA_ROOT
        with self.settings(MEDIA_ROOT=self.tmp_media_root):
            my_car = Car.objects.create(brand="Saab", name="9.3 2.0t")
            with open(os.path.join(old_media_root, "1st_Saab_9-3_SE.jpg"), mode="rb") as f:
                my_car.photo.save("some pic", ImageFile(f))
            self.assertEqual(my_car.photo.name, u'cars/Saab_9.3_2.0t.jpg')
            self.assertEqual(my_car.photo.size, 206420)

            # Change the photo and the name remains the same.
            with open(os.path.join(old_media_root, "Saab_9-3_Vector_sedan.jpg"), mode="rb") as f:
                my_car.photo.save("some pic", ImageFile(f))
            self.assertEqual(my_car.photo.name, u'cars/Saab_9.3_2.0t.jpg')
            self.assertEqual(my_car.photo.size, 384780)



# -*- coding: utf-8 -*-

from builtins import object
from builtins import str as text
from django.contrib import admin, messages
from django.contrib.admin.utils import label_for_field
from django.db.models import Q
from django.db.models.query import QuerySet
from django.db.models.fields import BooleanField
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.utils.http import urlencode, urlunquote
from django.utils.translation import gettext as _
from django_stachoutils.options import paginate
from django_stachoutils.views.actions import regroup_actions

ORDER_BY_ATTR = 'o'
ORDER_TYPE_ATTR = 'ot'

FILTER_IGNORE_DEFAULT = ['page', 'paginates_by', 'q', ORDER_BY_ATTR, ORDER_TYPE_ATTR]


def generic_change_response(request, url_continue, url_add_another, url_default):
    if "_continue" in request.POST:
        return HttpResponseRedirect(url_continue)
    elif "_addanother" in request.POST:
        return HttpResponseRedirect(url_add_another)
    else:
        return HttpResponseRedirect(url_default)


def generic_list(request, queryset, columns, template, model, ClassAdmin=None,
                 actions=[], filters=(), links=(), search=(), default_order=None,
                 paginate_by_default=100, paginates_by=[], qset_modifier=None,
                 qset_modifier_args=[], extra_params={}):
    """
     actions_par_groupe = [
        ('groupeA', [('action_1', 'libellé de l'action 1'),
                     ('action_3', 'libellé de l'action 3')]),
        ('groupeB', [('action_2', 'libellé de l'action 2')]),
     ]
    """
    list_id = model._meta.verbose_name
    set_columns_labels(model, columns)

    # Actions.
    actions = regroup_actions(actions)
    actions_par_groupe = []
    if ClassAdmin and actions:
        adm = ClassAdmin(model, admin.site)
        # {'invalider': (<unbound method ProgrammeAdmin.invalider>,
        #  'invalider', u'Invalider en...'), ...}
        admin_actions = adm.get_actions(request)
        for group, actions_in_group in actions:
            actions_dispos = [
                (callable(action) and action.__name__ or action,
                 action in admin_actions and admin_actions[action][2] or action.short_description)
                for action in actions_in_group if action in admin_actions or callable(action)
            ]
            if actions_dispos:
                actions_par_groupe.append((group, actions_dispos))

    # Exec action.
    if request.method == 'POST':
        action = request.POST.get('action')
        selected = request.POST.getlist('_selected_action')
        url_from = request.path + "?%s" % request.GET.urlencode()
        if not action:
            messages.add_message(request, messages.WARNING, "Veuillez sélectionner une action")
        if not selected:
            messages.add_message(request, messages.WARNING,
                                 "Veuillez sélectionner un ou plusieurs élements")
        if not action or not selected:
            return HttpResponseRedirect(url_from)

        # l'action est un callable.
        for group, actions_in_group in actions:
            for act in actions_in_group:
                if callable(act) and act.__name__ == action:
                    # On filtre les objets selectionnés si le queryset est un vrai queryset.
                    if not int(request.POST.get('select_across', 0)) and hasattr(queryset, 'filter'):
                        selected = request.POST.getlist(admin.ACTION_CHECKBOX_NAME)
                        queryset = queryset.filter(pk__in=selected)
                    response = act(None, request, queryset)
                    # response is actually a ResponseLike instance: TODO
                    if response:
                        return response
                    else:
                        return HttpResponseRedirect(url_from)

    # Si le queryset est une liste, on skip pas mal de choses.
    if not hasattr(queryset, 'filter'):
        c = {
            'object_list': queryset,
            'page': None,
            'get_params': dict(request.GET.items()),
            'actions': actions_par_groupe,
            'links': links,
            'path': request.path,
            'full_path': u'%s?%s' % (request.path, request.GET.urlencode()),
            'list_id': list_id,
        }
        c.update(extra_params)
        return render(request, template, c)

    # Filter columns according to the user permissions.
    columns = [col for col in columns if has_column_perm(request.user, col)]

    # Search.
    if request.GET.get('q'):
        query = request.GET['q']
        q_filters = [Q(**{s[1]: query}) for s in search]
        f = q_filters.pop()
        for filt in q_filters:
            f = f | filt
        queryset = queryset.filter(f)

    # Queryset.
    ordering = get_ordering_params(request, model, columns, default_order)
    if ordering:
        queryset = queryset.order_by(*ordering)

    # Apply filters.
    current_filters = get_current_filters(request, filters, model)
    queryset = filter_queryset(queryset, filters, current_filters, qset_modifier,
                               qset_modifier_args)
    count_across = queryset.count()
    page = paginate(request, queryset, paginate_by_default)

    # Filters.
    filters = [get_filter(model, current_filters, f) for f in filters]

    c = {
        'object_list': page.object_list,
        'page': page,
        'paginate_by': request.GET.get('paginate_by', paginate_by_default),
        'paginates_by': request.GET.get('paginates_by', paginates_by),
        'columns': columns,
        'actions': actions_par_groupe,
        'filters': filters,
        'links': links,
        'search': [s[0] for s in search],
        'get_params': dict(request.GET.items()),
        'path': request.path,
        'full_path': u'%s?%s' % (request.path, request.GET.urlencode()),
        'list_id': list_id,
        'count_across': count_across,
    }
    c.update(extra_params)
    return render(request, template, c)


def set_columns_labels(model, columns):
    for col in columns:
        if 'columns' in col:
            set_columns_labels(model, col['columns'])
        elif 'label' not in col:
            col['label'] = label_for_field(col['field'], model)


def get_ordering_params(request, model, columns, default_order):
    order_by, ordering = request.GET.get(ORDER_BY_ATTR, None), None

    if order_by:
        column = columns[int(order_by)]

        if 'ordering' in column:
            order_fields = column.get('ordering')
        else:
            attr_name = column['field']
            try:
                attr = getattr(model, attr_name)
            # Some fields are not reachable but we don't care,
            # we are looking for a method here.
            except AttributeError:
                attr = None
            if hasattr(attr, 'ordering'):
                order_fields = attr.ordering
            else:
                order_fields = [attr_name]

        order_type = ''
        if request.GET[ORDER_TYPE_ATTR] == 'desc':
            order_type = '-'

        ordering = ['%s%s' % (order_type, order_field) for order_field in order_fields]

    elif default_order:
        ordering = default_order

    return ordering


def has_column_perm(user, column):
    perms = column.get('perms')
    if not perms:
        return True
    for perm in perms:
        if perm[0] == '!':
            user_has_perm = not user.has_perm(perm[1:])
        else:
            user_has_perm = user.has_perm(perm)
        if not user_has_perm:
            return False
    return True


def filter_queryset(queryset, filters, current_filters, modifier, modifier_args):
    filter_attrs = {}
    filters_as_dict = dict([(f[0], f[1] if len(f) == 2 else {}) for f in filters])
    for key, value in current_filters.items():
        root_key = key.replace('__exact', '')
        filter_params = filters_as_dict.get(root_key)
        # filter can provide its own custom queryset filtering
        # or just be a parameter to the main filtering call.
        try:
            queryset = filter_params['queryset'](queryset, value)
        except (KeyError, TypeError):
            filter_attrs[key] = value

    queryset = queryset.filter(**filters_for_queryset(filter_attrs)).distinct()
    if modifier:
        queryset = modifier(queryset, *modifier_args)
    return queryset


def filters_for_queryset(current_filters):
    qset_filter_params = current_filters.copy()
    for k, v in current_filters.items():
        if k[-4:] == '__in':
            qset_filter_params[str(k)] = v.split(',')
    return qset_filter_params


def get_filter(model, current_filters, filter_params):
    """
    o filter_params may be a 1 elt or a 2 elts tuple

    Return a tuple:

    "filter title",
    [
        (label, url, is_selected),  # filter value #1
        (label, url, is_selected),  # filter value #2
        ...
    ]
    """

    params = {
        'filter_key': filter_params[0],
        'title': None,
        'queryset': None,
        'choices': None,
        'empty_choice': False,
        'filter_test': 'exact',
    }
    try:
        params.update(filter_params[1])
    except IndexError:
        pass

    choices, empty_choice = params['choices'], params['empty_choice']
    filter_key, filter_test = params['filter_key'], params['filter_test']
    title = params['title']

    field, attrib = None, None

    def discover_field(model, filter_key):
        attrs = filter_key.split('__')
        rel_models = attrs[:-1]
        attr = attrs[-1]
        mod = model
        for rel_model in rel_models:
            mod = mod._meta.get_field(rel_model).remote_field.model
        field = mod._meta.get_field(attr)

        return field, attr

    if not title:
        field, attrib = discover_field(model, filter_key)
        title = field.verbose_name

    if choices:
        FilterKlass = FilterSimple
        if type(choices) == QuerySet:
            FilterKlass = Filter
    else:
        if not field:
            field, attrib = discover_field(model, filter_key)
        if type(field) == BooleanField:
            FilterKlass = FilterSimple
            choices = [
                ('1', 'Oui'),
                ('0', 'Non'),
            ]
        else:
            FilterKlass = Filter
            choices = field.remote_field.model.objects.all()

    def get_displayed_choices(FilterKlass, choices, current_filters, filter_test,
                              filter_key, model, attrib):
        displayed_choices = [
            (lambda f=FilterKlass(o, attrib, filter_key, filter_test, current_filters): (
                f.url, f.label, f.is_selected))()
            for o in choices
        ]
        displayed_choices.insert(0, (lambda f=FilterAll(attrib, filter_key, filter_test, current_filters): (
            f.url, f.label, f.is_selected))())
        if empty_choice:
            displayed_choices.append((lambda f=FilterNone(model, attrib, filter_key, filter_test, current_filters): (
                f.url, f.label, f.is_selected))())

        return displayed_choices

    displayed_choices = get_displayed_choices(FilterKlass, choices, current_filters,
                                              filter_test, filter_key, model, attrib)

    return title, displayed_choices


def get_current_filters(request, filters, model, ignore=FILTER_IGNORE_DEFAULT):
    """ Return {'key': 'value', ...} """

    filters_keys = [f[0] for f in filters]

    current_filters = {}
    for k, v in request.GET.items():
        if k.replace('__exact', '') in filters_keys or is_a_filter(model, k):
            current_filters[str(k)] = str(urlunquote(v))
    return current_filters


##

LOOKUPS = [
    'exact',
    'iexact',
    'contains',
    'icontains',
    'in',
    'gt',
    'gte',
    'lt',
    'lte',
    'startswith',
    'istartswith',
    'endswith',
    'iendswith',
    'range',
    'year',
    'month',
    'day',
    'week_day',
    'isnull',
    'search',
    'regex',
    'iregex',
]


def is_a_filter(model, filtr):
    instance = model()
    lst = filtr.split('__')
    if lst[-1] in LOOKUPS:
        del lst[-1]
    if hasattr(instance, lst[0]):
        return True


class Filter(object):
    """An object to represente the filters in the index pages. Works on FK. """

    def __init__(self, instance, attr, filter_string, test, current_filters):
        if instance:
            self.instance = instance
            self.label = text(instance)
            self.value = str(instance.pk)
        self.current_filters = current_filters
        self.key = '%s__%s' % (filter_string, test)
        self.isnull = '%s__isnull' % (filter_string,)
        self._url = None
        self._is_selected = None

    def _get_url(self):
        """L'url d'un filtre doit conserver les filtres en cours pour être additifs. """
        if self._url is None:
            filters = self.get_base_filters()
            out = filters.copy()
            for k, v in filters.items():
                if not v:
                    del out[k]  # del empty filters
            self._url = urlencode(sorted((k, v) for k, v in out.items()))  # sorted to pass the tests...
        return self._url
    url = property(_get_url)

    def get_base_filters(self):
        filters = self.current_filters.copy()
        # on update le parametre si deja present dans l'url.
        filters.update({self.key: self.value})
        # on vire l'eventuel filtre is_null.
        if self.isnull in filters:
            del filters[self.isnull]
        return filters

    def _get_is_selected(self):
        """L'url d'un filtre doit conserver les filtres en cours pour être additifs. """
        if self._is_selected is None:
            selected = self.current_filters.get(self.key, None)
            self._is_selected = (self.value == selected)
        return self._is_selected
    is_selected = property(_get_is_selected)


class FilterSimple(Filter):
    def __init__(self, choice, attr, filter_string, test, current_filters):
        super(FilterSimple, self).__init__(None, attr, filter_string, test, current_filters)
        self.label = choice[1]
        self.value = str(choice[0])


class FilterAll(Filter):
    """Désactive le filtre en selectionnant tous les objects. """

    def __init__(self, attr, filter_string, test, current_filters):
        super(FilterAll, self).__init__(None, attr, filter_string, test, current_filters)
        self.label = _("All")
        self.value = None

    def _get_is_selected(self):
        if self._is_selected is None:
            if self.key not in self.current_filters and self.isnull not in self.current_filters:
                self._is_selected = True
        return self._is_selected
    is_selected = property(_get_is_selected)

    def get_base_filters(self):
        """L'url est construite à partir de l'url en cours, en supp l'eventuel filtre sur le groupe. """
        filters = super(FilterAll, self).get_base_filters()
        filters.pop(self.key, None)
        return filters


class FilterNone(Filter):
    def __init__(self, model, attr, filter_string, test, current_filters):
        super(FilterNone, self).__init__(None, attr, filter_string, 'isnull', current_filters)
        self.sibling_key = '%s__%s' % (filter_string, test)  # key for actual filtering
        self.label = u'Aucun'
        self.value = True

    def _get_is_selected(self):
        if self.isnull in self.current_filters:
            self._is_selected = True
        return self._is_selected
    is_selected = property(_get_is_selected)

    def get_base_filters(self):
        filters = super(FilterNone, self).get_base_filters()
        filters.update({self.key: self.value})
        if self.sibling_key in filters:
            del filters[self.sibling_key]
        return filters



#!/usr/bin/env python
#
# Inspired from https://github.com/django/django/blob/master/tests/runtests.py.
#
from argparse import ArgumentParser
import os
import sys

import django
from django.apps import apps
from django.conf import settings
from django.test.utils import get_runner

TEST_TEMPLATE_DIR = 'templates'

RUNTESTS_DIR = os.path.abspath(os.path.dirname(__file__))

SUBDIRS_TO_SKIP = [
    'data',
]

ALWAYS_INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'django.contrib.sites',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.admin',
    'django.contrib.staticfiles',
    'django_stachoutils',
    'sorl.thumbnail',
]

ALWAYS_MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

MIDDLEWARE_ATTR = "MIDDLEWARE"


def get_test_modules():
    modules = []
    for f in os.listdir(RUNTESTS_DIR):
        if ('.' in f or
                os.path.basename(f) in SUBDIRS_TO_SKIP or
                os.path.isfile(f) or
                not os.path.exists(os.path.join(f, '__init__.py'))):
            continue
        modules.append(f)
    return modules


def setup(verbosity, test_labels):
    state = {
        'INSTALLED_APPS': settings.INSTALLED_APPS,
        'ROOT_URLCONF': getattr(settings, "ROOT_URLCONF", ""),
        'TEMPLATES': settings.TEMPLATES,
        'LANGUAGE_CODE': settings.LANGUAGE_CODE,
        'STATIC_URL': settings.STATIC_URL,
        'STATIC_ROOT': settings.STATIC_ROOT,
        MIDDLEWARE_ATTR: getattr(settings, MIDDLEWARE_ATTR),
    }

    # Redirect some settings for the duration of these tests.
    settings.INSTALLED_APPS = ALWAYS_INSTALLED_APPS
    settings.ROOT_URLCONF = 'urls'
    settings.STATIC_URL = '/static/'
    settings.STATIC_ROOT = ''
    settings.TEMPLATES = [{
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(RUNTESTS_DIR, TEST_TEMPLATE_DIR)],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
            ],
        },

    }]
    settings.LANGUAGE_CODE = 'en'
    settings.SITE_ID = 1
    setattr(settings, MIDDLEWARE_ATTR, ALWAYS_MIDDLEWARE_CLASSES)
    # Ensure the middleware classes are seen as overridden otherwise we get a compatibility warning.
    settings._explicit_settings.add(MIDDLEWARE_ATTR)
    settings.MIGRATION_MODULES = {
        'auth': None,
        'contenttypes': None,
        'sessions': None,
        'autocomplete': None,
    }

    # Load all the ALWAYS_INSTALLED_APPS.
    django.setup()

    # Load the test model apps.
    if not test_labels:
        modules = get_test_modules()
    else:
        modules = set([label.split(".")[0] for label in test_labels])

    for module_label in modules:
        settings.INSTALLED_APPS.append(module_label)
    apps.set_installed_apps(settings.INSTALLED_APPS)

    return state


def teardown(state):
    # Restore the old settings.
    for key, value in state.items():
        setattr(settings, key, value)


def django_tests(verbosity, test_labels):
    state = setup(verbosity, test_labels)

    if not hasattr(settings, 'TEST_RUNNER'):
        settings.TEST_RUNNER = 'django.test.runner.DiscoverRunner'
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=verbosity)
    failures = test_runner.run_tests(test_labels or get_test_modules())

    teardown(state)
    return failures


if __name__ == "__main__":
    parser = ArgumentParser(description="Run the Django test suite.")
    parser.add_argument(
        'modules', nargs='*', metavar='module',
        help='Optional path(s) to test modules; e.g. "autocomplete" or '
        '"utils.tests.UtilsTest.test_clean_fieldname_prefix".')
    parser.add_argument(
        '-v', '--verbosity', default=1, type=int, choices=[0, 1, 2, 3],
        help='Verbosity level; 0=minimal output, 1=normal output, 2=all output')
    parser.add_argument(
        '--liveserver',
        help='Overrides the default address where the live server (used with '
             'LiveServerTestCase) is expected to run from. The default value '
             'is localhost:8081.')
    options = parser.parse_args()

    # Allow including a trailing slash on app_labels for tab completion convenience
    options.modules = [os.path.normpath(labels) for labels in options.modules]

    if options.liveserver is not None:
        os.environ['DJANGO_LIVE_TEST_SERVER_ADDRESS'] = options.liveserver

    os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'
    failures = django_tests(options.verbosity, options.modules)
    if failures:
        sys.exit(bool(failures))



from django.db import models


class Car(models.Model):
    name = models.CharField(max_length=100)
    brand = models.CharField(max_length=100)

    def __str__(self):
        return self.name


class CarOption(models.Model):
    car = models.ForeignKey('Car', on_delete=models.CASCADE)
    name = models.CharField(max_length=15)

    def __str__(self):
        return self.name



# -*- coding: utf-8 -*-

from django.contrib.contenttypes.models import ContentType
from django.contrib.admin.models import LogEntry, ADDITION, CHANGE, DELETION
from django.utils.encoding import force_str

from django.core.paginator import Paginator, InvalidPage, EmptyPage


def log_addition(request, object):
    LogEntry.objects.log_action(
        user_id         = request.user.pk,
        content_type_id = ContentType.objects.get_for_model(object).pk,
        object_id       = object.pk,
        object_repr     = force_str(object),
        action_flag     = ADDITION
    )


def log_change(request, object, message):
    LogEntry.objects.log_action(
        user_id         = request.user.pk,
        content_type_id = ContentType.objects.get_for_model(object).pk,
        object_id       = object.pk,
        object_repr     = force_str(object),
        action_flag     = CHANGE,
        change_message  = message
    )


def log_deletion(request, object, object_repr):
    LogEntry.objects.log_action(
        user_id         = request.user.id,
        content_type_id = ContentType.objects.get_for_model(object).pk,
        object_id       = object.pk,
        object_repr     = object_repr,
        action_flag     = DELETION
    )


def log_change_for_user(user, object, message):
    LogEntry.objects.log_action(
        user_id         = user.pk,
        content_type_id = ContentType.objects.get_for_model(object).pk,
        object_id       = object.pk,
        object_repr     = force_str(object),
        action_flag     = CHANGE,
        change_message  = message
    )


def logs_for_formsets(mode, request, object, forms):
    """forms est une liste de forms ou formsets."""
    if mode == 'addition':
        log_addition(request, object)
    elif mode == 'change':
        msg = u'a modifié : '
        for f in forms:
            msg += _log_for_forms(f)
        log_change(request, object, msg)


def _log_for_forms(f):
    msg = ''
    if hasattr(f, 'changed_data'):
        if f.changed_data:
            msg += u', '.join(f.changed_data)
            if hasattr(f, 'instance'):
                msg += u' (%s [%s]) ; ' % (f.instance._meta.verbose_name, f.instance)
            else:
                msg += u' (%s) ; ' % (f.__class__.__name__)
    if hasattr(f, 'forms'):
        for f2 in f.forms:
            msg += _log_for_forms(f2)
    return msg



def paginate(request, queryset, paginate_by=50):
    if request.GET.get('paginate_by') == 'All':
        paginate_by = queryset.count()
    elif 'paginate_by' in request.GET:
        try:
            paginate_by = int(request.GET['paginate_by'])
        except ValueError:
            pass  # on retombe sur la valeur par defaut.

    paginator = Paginator(queryset, paginate_by)
    try:
        num_page = int(request.GET.get('page', '1'))
    except ValueError:
        num_page = 1
    try:
        page = paginator.page(num_page)
    except (EmptyPage, InvalidPage):
        page = paginator.page(paginator.num_pages)

    return page



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-18 09:39
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Band',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='BandMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=100)),
                ('last_name', models.CharField(max_length=100)),
                ('story', models.TextField(blank=True, null=True)),
                ('band', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='my_app.Band')),
            ],
        ),
        migrations.CreateModel(
            name='Instrument',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='MusicGenre',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True)),
            ],
        ),
        migrations.AddField(
            model_name='bandmember',
            name='favorite_instrument',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='my_app.Instrument'),
        ),
        migrations.AddField(
            model_name='band',
            name='genre',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='my_app.MusicGenre'),
        ),
        migrations.AlterUniqueTogether(
            name='bandmember',
            unique_together=set([('first_name', 'last_name')]),
        ),
    ]



# -*- coding: utf-8 -*-

from django import forms
from django.test import TestCase

from django_stachoutils.forms import NestedModelForm
from .models import Car, Person


class PersonForm(forms.ModelForm):
    class Meta:
        model = Person
        fields = ('name', 'lastname', 'gender')


class CarForm(NestedModelForm):
    owner = forms.ModelChoiceField(queryset=Person.objects.all(),
                                   initial=None, required=False,
                                   widget=forms.HiddenInput())
    class Meta:
        model = Car
        fields = ('name', 'brand', 'owner')

    class Nested:
        form = PersonForm
        fk = 'owner'


class NestedModelFormTest(TestCase):
    def test_render_nested_model_form(self):
        form = CarForm()
        self.assertHTMLEqual(
            form.as_table(),
            """
            <tr>
              <th><label for="id_name">Name:</label></th>
              <td><input type="text" name="name" required id="id_name" maxlength="20" /></td>
            </tr>
            <tr>
              <th><label for="id_brand">Brand:</label></th>
              <td>
                <input type="text" name="brand" required id="id_brand" maxlength="100" />
                <input id="id_owner" name="owner" type="hidden" />
              </td>
            </tr>
            """
        )
        self.assertHTMLEqual(
            form.nested_form.as_table(),
            """
            <tr>
              <th><label for="id_None_OWNER-name">Name:</label></th>
              <td><input type="text" name="None_OWNER-name" required id="id_None_OWNER-name" maxlength="100" /></td>
            </tr>
            <tr>
              <th><label for="id_None_OWNER-lastname">Lastname:</label></th>
              <td><input type="text" name="None_OWNER-lastname" required id="id_None_OWNER-lastname" maxlength="100" /></td>
            </tr>
            <tr>
              <th><label for="id_None_OWNER-gender">Gender:</label></th>
              <td><select name="None_OWNER-gender" required id="id_None_OWNER-gender">
                    <option value="" selected>---------</option>
                    <option value="male">Male</option>
                    <option value="female">Female</option>
                  </select>
              </td>
            </tr>
            """
        )

    def test_bound_nested_model_form(self):
        form = CarForm({
            'name': '9.3 2.0t',
            'brand': 'Saab',
            'owner': '',
        })
        self.assertTrue(form.is_valid())
        form.save()
        my_car = Car.objects.latest('pk')
        self.assertEqual(my_car.brand, 'Saab')
        self.assertEqual(my_car.name, '9.3 2.0t')

        # Data hasn't changed.
        form = CarForm({
            'name': '9.3 2.0t',
            'brand': 'Saab',
            'owner': '',
        }, instance=my_car)
        self.assertFalse(form.has_changed())

        # Data has changed.
        form = CarForm({
            'name': '9.3 2.0T Biopower',
            'brand': 'Saab',
            'owner': '',
        }, instance=my_car)
        self.assertTrue(form.has_changed())

        form.save()
        my_car = Car.objects.get(pk=my_car.pk)
        self.assertEqual(my_car.brand, 'Saab')
        self.assertEqual(my_car.name, '9.3 2.0T Biopower')

        # With nested data.
        form = CarForm({
            'name': '900 Turbo 16',
            'brand': 'Saab',
            'owner': '',
            'None_OWNER-name': 'Stan',
            'None_OWNER-lastname': 'Guerra',
            'None_OWNER-gender': 'male',
        })
        self.assertTrue(form.is_valid())
        form.save()
        my_car = Car.objects.latest('pk')
        self.assertEqual(my_car.brand, 'Saab')
        self.assertEqual(my_car.name, '900 Turbo 16')
        self.assertEqual(my_car.owner.name, 'Stan')

    def test_bound_nested_model_form_with_invalid_data(self):
        form = CarForm({
            'name': 'Nine Three Two-liters Turbocharged',
            'brand': 'Saab',
            'owner': '',
        })
        self.assertFalse(form.is_valid())
        self.assertEqual(form.non_nested_errors, {'name':['Ensure this value has at most 20 characters (it has 34).']})

    def test_prefixed_bound_nested_model_form(self):
        form = CarForm({
            'car-name': '9.3 2.0t',
            'car-brand': 'Saab',
            'car-owner': '',
        }, prefix='car')
        self.assertTrue(form.is_valid())
        form.save()
        my_car = Car.objects.latest('pk')
        self.assertEqual(my_car.brand, 'Saab')
        self.assertEqual(my_car.name, '9.3 2.0t')

        form = CarForm({
            'car-name': '900 Turbo 16',
            'car-brand': 'Saab',
            'car-owner': '',
            'car_OWNER-name': 'Stan',
            'car_OWNER-lastname': 'Guerra',
            'car_OWNER-gender': 'male',
        }, prefix='car')
        self.assertTrue(form.is_valid())
        form.save()
        my_car = Car.objects.latest('pk')
        self.assertEqual(my_car.brand, 'Saab')
        self.assertEqual(my_car.name, '900 Turbo 16')
        self.assertEqual(my_car.owner.name, 'Stan')



# -*- coding: utf-8 -*-



def regroup_actions(actions_flat):
    regrouped_actions_dict = {}
    for action in actions_flat:
        group = hasattr(action, "group") and getattr(action, "group") or "autres"
        actions_in_group = regrouped_actions_dict.setdefault(group, [])
        actions_in_group.append(action)
    last_group = ("autres" in regrouped_actions_dict) and regrouped_actions_dict.pop("autres") or None

    return [(group, regrouped_actions_dict[group]) for group in sorted(regrouped_actions_dict.keys())] + \
            (last_group and [("autres", last_group)] or [])



# -*- coding: utf-8 -*-

import os
from datetime import datetime
from django.core.files.base import ContentFile
from django.conf import settings
from django.test import SimpleTestCase
from pathlib import Path

import django_stachoutils as utils


class UtilsTest(SimpleTestCase):
    def test_format_datetime(self):
        self.assertEqual(utils.format_datetime(datetime(2017, 1, 31, 16, 58, 33)), u'Jan. 31, 2017, 4:58 p.m.')
        self.assertEqual(utils.format_datetime(datetime(2017, 1, 31, 16, 58, 33), time=False), u'Jan. 31, 2017')
        self.assertEqual(utils.format_datetime(datetime(2017, 1, 31, 16, 58, 33), date=False), u'4:58 p.m.')

    def test_format_number(self):
        self.assertEqual(utils.format_number(2312), "2,312")
        self.assertEqual(utils.format_number(2312, u" "), "2 312")
        self.assertEqual(utils.format_number(), "")

    def test_truncate_chars(self):
        self.assertEqual(utils.truncate_chars("A quick brown fox jumps", 7), "A quick...")
        self.assertEqual(utils.truncate_chars(u"¿ Holà señor, como estas ?", 12), u"¿ Holà señor...")

    def test_strip_accents(self):
        self.assertEqual(utils.strip_accents(u"Midi-Pyrénées"), u'Midi-Pyrenees')

    def test_format_xml_entity(self):
        self.assertEqual(utils.format_xml_entity(u"Côtes d'Armor"), b'COTES-D-ARMOR')

    def test_latin1_safe_xml_encode(self):
        self.assertEqual(utils.latin1_safe_xml_encode(u"ϟϟ"), '&#991;&#991;')

    def test_clean_lettrine(self):
        self.assertEqual(utils.clean_lettrine(u"P aris. Superbe appartement"),
                         u"Paris. Superbe appartement")
        self.assertEqual(utils.clean_lettrine(u"A 2 pas de la Seine"),
                         u"A 2 pas de la Seine")

    def test_int_to_roman(self):
        self.assertEqual(utils.int_to_roman(13), u"XIII")

    def test_filter_non_printable(self):
        self.assertEqual(utils.filter_non_printable(u"This is ϞϞ !\x04"), u"This is ϞϞ !")

    def test_camelize(self):
        self.assertEqual(utils.camelize(u"this is a string"), u"This Is A String")

    def test_urldecode(self):
        self.assertEqual(utils.urldecode(u"?foo=1&bar=2"), {u"foo": u"1", u"bar": u"2"})

    def test_files_are_equal(self):
        image1 = os.path.join(settings.MEDIA_ROOT, 'jardin.jpg')
        image2 = os.path.join(settings.MEDIA_ROOT, 'terrasse.jpg')
        f1 = ContentFile(Path(image1).read_bytes())
        f2 = ContentFile(Path(image2).read_bytes())

        # Files are differents by size
        self.assertFalse(utils.files_are_equal(f1, f2))
        self.assertNotEqual(f1.size, f2.size)

        # Files are equals
        f1bis = ContentFile(Path(image1).read_bytes())
        self.assertTrue(utils.files_are_equal(f1, f1bis))

        # Files are differents by content
        f1_altered = ContentFile(Path(image1).read_bytes().replace(b'\x14', b'\xff'))
        self.assertFalse(utils.files_are_equal(f1, f1_altered))
        self.assertEqual(f1.size, f1_altered.size)

    def test_simple_decorator(self):
        @utils.simple_decorator
        def mess_with_the_data(func):
            def new_function(a, b):
                return func(a**2, b**2)
            return new_function

        @mess_with_the_data
        def addition(a, b):
            """A basic mathematical function"""
            return a + b

        self.assertEqual(addition(2, 3), 13)
        self.assertEqual(addition.__doc__, 'A basic mathematical function')
        self.assertEqual(addition.__name__, 'addition')



# -*- coding: utf-8 -*-

import datetime
from django.contrib.auth.models import Permission, User
from django.contrib.contenttypes.models import ContentType
from django.contrib.messages.storage.base import BaseStorage
from django.test import RequestFactory, TestCase
from django_stachoutils.views import generic
from .actions import sell_cars
from .admin import CarAdmin
from .models import Car, Company, Garage, Person


class MessageStorage(BaseStorage):
    """This is a basic messages storage since RequestFactory does not provide
       a complete request object according to settings.MIDDLEWARE.
    """
    def __init__(self, request, *args, **kwargs):
        self.messages = None
        super(MessageStorage, self).__init__(request, *args, **kwargs)

    def _get(self, *args, **kwargs):
        return self.messages, True

    def _store(self, messages, response, *args, **kwargs):
        if messages:
            self.messages = messages
        else:
            self.messages = None
        return []


class BaseGenericListTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.c1 = Car.objects.create(brand='Saab', name='9.3', price=12500, purchased_on=datetime.date(2015, 7, 29))
        cls.c2 = Car.objects.create(brand='Saab', name='900', price=1800, purchased_on=datetime.date(2001, 3, 29))
        cls.c3 = Car.objects.create(brand='Saab', name='9.5', price=23700, purchased_on=datetime.date(2012, 2, 11))
        cls.c4 = Car.objects.create(brand='Alfa-Romeo', name='Giullia', price=19500, purchased_on=datetime.date(2014, 7, 12))
        cls.c5 = Car.objects.create(brand='Alfa-Romeo', name='Sprint', price=11200, purchased_on=datetime.date(2015, 9, 29))
        cls.c6 = Car.objects.create(brand='Subaru', name='Forester', price=34500, purchased_on=datetime.date(2016, 1, 29))
        cls.c7 = Car.objects.create(brand='Subaru', name='Impreza', price=11200, purchased_on=datetime.date(2012, 6, 29))

        cls.user = User.objects.create_user('john', 'lennon@thebeatles.com', 'johnpassword')

    def setUp(self):
        self.queryset = Car.objects.all().order_by('brand', 'name')
        self.rf = RequestFactory()
        self.actions = [sell_cars]
        self.columns = [
            {'label': 'Name', 'field': 'name', 'link': {'href': 'get_absolute_url'}},
            {'label': 'Brand', 'field': 'brand'},
            {'label': 'Price', 'field': 'price_html'},
            {'label': 'Purchased Year', 'field': 'purchased_on', 'filters': [('date', 'Y')]},
            {'label': 'For Sale', 'field': 'for_sale'},
        ]


class UnitTest(TestCase):
    def setUp(self):
        self.rf = RequestFactory()
        self.columns = [
            {'field': 'name', 'link': {'href': 'get_absolute_url'}, 'ordering': ['brand', 'name']},
            {'field': 'brand'},
            {'field': 'price_html'},
            {'field': 'purchased_on', 'filters': [('date', 'Y')]},
            {'field': 'for_sale'},
        ]

    def test_set_columns_labels(self):
        generic.set_columns_labels(Car, self.columns)
        self.assertEqual([c['label'] for c in self.columns],
                         [u'name', u'brand', u'price', u'purchased on', u'for sale'])

    def test_recursion_in_set_columns_labels(self):
        columns = [
            {'field': 'name', 'link': {'href': 'get_absolute_url'}},
            {'field': 'brand'},
            {'field': 'price_html'},
            {'field': 'purchased_on', 'filters': [('date', 'Y')]},
            {'field': 'for_sale'},
            {
                'with': 'description_by_lang',
                'columns': (
                    {'label': u'Description (Fr)', 'field': 'fr'},  # This should call set_columns_labels() if no label
                    {'label': u'Description (En)', 'field': 'en'},  # is provided but still I can't see a use case...
                )
            },
        ]
        generic.set_columns_labels(Car, columns)

    def test_get_ordering_params(self):
        request = self.rf.get('/cars?page=2&o=1&ot=asc')  # Order by Brand
        ordering = generic.get_ordering_params(request, Car, self.columns, default_order=None)
        self.assertEqual(ordering, ['brand'])

        request = self.rf.get('/cars?page=2&o=1&ot=desc')  # Descending order
        ordering = generic.get_ordering_params(request, Car, self.columns, default_order=None)
        self.assertEqual(ordering, ['-brand'])

    def test_get_ordering_params_by_method(self):
        request = self.rf.get('/cars?page=2&o=2&ot=asc')  # Order by Price, then name (price_html method)
        ordering = generic.get_ordering_params(request, Car, self.columns, default_order=None)
        self.assertEqual(ordering, ['price', 'name'])

        request = self.rf.get('/cars?page=2&o=2&ot=desc')  # Descending order
        ordering = generic.get_ordering_params(request, Car, self.columns, default_order=None)
        self.assertEqual(ordering, ['-price', '-name'])

    def test_get_ordering_params_default(self):
        request = self.rf.get('/cars?page=2')
        ordering = generic.get_ordering_params(request, Car, self.columns, default_order='name')
        self.assertEqual(ordering, 'name')

    def test_get_ordering_params_with_ordering_set_in_column(self):
        request = self.rf.get('/cars?page=2&o=0&ot=asc')  # Order by name
        ordering = generic.get_ordering_params(request, Car, self.columns, default_order=None)
        self.assertEqual(ordering, ['brand', 'name'])

        request = self.rf.get('/cars?page=2&o=0&ot=desc')  # Descending
        ordering = generic.get_ordering_params(request, Car, self.columns, default_order=None)
        self.assertEqual(ordering, ['-brand', '-name'])

    def test_has_column_perm(self):
        user = User.objects.create_user('john', 'lennon@thebeatles.com', 'johnpassword')
        col = {'field': 'purchased_on',
               'perms': ['views.see_car_purchased_on']}
        self.assertFalse(generic.has_column_perm(user, col))

        # Add the permission to the user
        car_content_type = ContentType.objects.get_for_model(Car)
        perm = Permission.objects.create(name="Can see puchased on date of a car",
                                         content_type=car_content_type,
                                         codename='see_car_purchased_on')
        user.user_permissions.add(perm)
        user = User.objects.get(pk=user.pk)
        self.assertTrue(generic.has_column_perm(user, col))

        # Test the 'not' operator
        col = {'field': 'price',
               'perms': ['!views.see_car_purchased_on']}
        self.assertFalse(generic.has_column_perm(user, col))

    def test_get_filter(self):
        current_filters = {'brand': 'Saab'}

        filter_1 = ('for_sale', )
        self.assertEqual(generic.get_filter(Car, current_filters, filter_1), (
            u'for sale', [
                ('brand=Saab', u'All', True),
                ('brand=Saab&for_sale__exact=1', 'Oui', False),
                ('brand=Saab&for_sale__exact=0', 'Non', False)
            ]
        ))

    def test_get_filter_fk(self):
        current_filters = {'brand': 'Saab'}

        Company.objects.create(name='Thomson Reuters')
        Company.objects.create(name='Id Software')
        Company.objects.create(name='Industrial Light & Magic', short_name='ILM')

        filter_2 = ('last_driver__employed_by', )
        self.assertEqual(generic.get_filter(Car, current_filters, filter_2), (
            u'employed by', [
                ('brand=Saab', u'All', True),
                ('brand=Saab&last_driver__employed_by__exact=1', u'Thomson Reuters', False),
                ('brand=Saab&last_driver__employed_by__exact=2', u'Id Software', False),
                ('brand=Saab&last_driver__employed_by__exact=3', u'ILM', False)
            ]
        ))

        # Set the title
        filter_3 = ('last_driver__employed_by', {'title': 'Company'})
        self.assertEqual(generic.get_filter(Car, current_filters, filter_3), (
            u'Company', [
                ('brand=Saab', u'All', True),
                ('brand=Saab&last_driver__employed_by__exact=1', u'Thomson Reuters', False),
                ('brand=Saab&last_driver__employed_by__exact=2', u'Id Software', False),
                ('brand=Saab&last_driver__employed_by__exact=3', u'ILM', False)
            ]
        ))

    def test_get_filter_through_related(self):
        current_filters = {'owner': 'Luigi'}

        Company.objects.create(name='Thomson Reuters')
        Company.objects.create(name='Id Software')
        Company.objects.create(name='Industrial Light & Magic', short_name='ILM')

        filter_1 = ('car__last_driver__employed_by', )
        self.assertEqual(generic.get_filter(Garage, current_filters, filter_1), (
            u'employed by', [
                ('owner=Luigi', u'All', True),
                ('car__last_driver__employed_by__exact=1&owner=Luigi', u'Thomson Reuters', False),
                ('car__last_driver__employed_by__exact=2&owner=Luigi', u'Id Software', False),
                ('car__last_driver__employed_by__exact=3&owner=Luigi', u'ILM', False)
            ]
        ))

        # Set the title
        filter_1 = ('car__last_driver__employed_by', {'title': 'Partners'})
        self.assertEqual(generic.get_filter(Garage, current_filters, filter_1), (
            u'Partners', [
                ('owner=Luigi', u'All', True),
                ('car__last_driver__employed_by__exact=1&owner=Luigi', u'Thomson Reuters', False),
                ('car__last_driver__employed_by__exact=2&owner=Luigi', u'Id Software', False),
                ('car__last_driver__employed_by__exact=3&owner=Luigi', u'ILM', False)
            ]
        ))



class GenericChangeResponseTest(TestCase):
    url_continue = '/cars/1/'
    url_add_another = '/cars/add/'
    url_default = '/cars/'

    def setUp(self):
        self.rf = RequestFactory()

    def test_generic_change_response(self):
        request = self.rf.post('/cars/1/')
        response = generic.generic_change_response(request, self.url_continue,
                                                   self.url_add_another, self.url_default)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.url_default)

    def test_generic_change_response_continue(self):
        request = self.rf.post('/cars/1/', {'_continue': '1'})
        response = generic.generic_change_response(request, self.url_continue,
                                                   self.url_add_another, self.url_default)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.url_continue)

    def test_generic_change_response_add_another(self):
        request = self.rf.post('/cars/1/', {'_addanother': '1'})
        response = generic.generic_change_response(request, self.url_continue,
                                                   self.url_add_another, self.url_default)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.url_add_another)


class GenericListTest(BaseGenericListTest):
    template = 'generic_list_test.html'

    def test_generic_list(self):
        request = self.rf.get('/cars?page=2')
        request.user = self.user
        response = generic.generic_list(request, self.queryset, self.columns, self.template,
                                        Car, ClassAdmin=CarAdmin, actions=self.actions)
        self.assertHTMLEqual(response.content.decode('utf8'), """
            <html>
              <form action="" method="post">
                <div class="actions">
                  <label>Action&nbsp;:
                    <select name="action">
                      <option selected="selected" value="">---------</option>
                      <optgroup label="autres">
                        <option value="sell_cars">Sell the cars</option>
                      </optgroup>
                    </select>
                  </label>
                  <button value="0" name="index" title="Execute the selected action" class="button" type="submit">Execute</button>
                </div>
              </form>
              <div>
                <div><input type="checkbox" name="_selected_action" value="%(al1)d" />Alfa-Romeo - Giullia</div>
                <div><input type="checkbox" name="_selected_action" value="%(al2)d" />Alfa-Romeo - Sprint</div>
                <div><input type="checkbox" name="_selected_action" value="%(sa1)d" />Saab - 9.3</div>
                <div><input type="checkbox" name="_selected_action" value="%(sa2)d" />Saab - 9.5</div>
                <div><input type="checkbox" name="_selected_action" value="%(sa3)d" />Saab - 900</div>
                <div><input type="checkbox" name="_selected_action" value="%(su1)d" />Subaru - Forester</div>
                <div><input type="checkbox" name="_selected_action" value="%(su2)d" />Subaru - Impreza</div>
              <div>
            </html>""" % {
                'al1': self.c4.pk,
                'al2': self.c5.pk,
                'sa1': self.c1.pk,
                'sa2': self.c3.pk,
                'sa3': self.c2.pk,
                'su1': self.c6.pk,
                'su2': self.c7.pk,
            })

    def test_ordered_generic_list(self):
        request = self.rf.get('/cars?page=2')
        request.user = self.user
        response = generic.generic_list(request, self.queryset, self.columns, self.template,
                                        Car, ClassAdmin=CarAdmin, actions=self.actions,
                                        default_order=['name', 'brand'])
        self.assertHTMLEqual(response.content.decode('utf8'), """
            <html>
              <form action="" method="post">
                <div class="actions">
                  <label>Action&nbsp;:
                    <select name="action">
                      <option selected="selected" value="">---------</option>
                      <optgroup label="autres">
                        <option value="sell_cars">Sell the cars</option>
                      </optgroup>
                    </select>
                  </label>
                  <button value="0" name="index" title="Execute the selected action" class="button" type="submit">Execute</button>
                </div>
              </form>
              <div>
                <div><input type="checkbox" name="_selected_action" value="%(sa1)d" />Saab - 9.3</div>
                <div><input type="checkbox" name="_selected_action" value="%(sa2)d" />Saab - 9.5</div>
                <div><input type="checkbox" name="_selected_action" value="%(sa3)d" />Saab - 900</div>
                <div><input type="checkbox" name="_selected_action" value="%(su1)d" />Subaru - Forester</div>
                <div><input type="checkbox" name="_selected_action" value="%(al1)d" />Alfa-Romeo - Giullia</div>
                <div><input type="checkbox" name="_selected_action" value="%(su2)d" />Subaru - Impreza</div>
                <div><input type="checkbox" name="_selected_action" value="%(al2)d" />Alfa-Romeo - Sprint</div>
              <div>
            </html>""" % {
                'al1': self.c4.pk,
                'al2': self.c5.pk,
                'sa1': self.c1.pk,
                'sa2': self.c3.pk,
                'sa3': self.c2.pk,
                'su1': self.c6.pk,
                'su2': self.c7.pk,
            })

    def test_generic_list_with_object_list(self):
        request = self.rf.get('/cars?page=2')
        request.user = self.user
        response = generic.generic_list(request, list(self.queryset), self.columns, self.template,
                                        Car, ClassAdmin=CarAdmin, actions=self.actions)
        self.assertHTMLEqual(response.content.decode('utf8'), """
            <html>
              <form action="" method="post">
                <div class="actions">
                  <label>Action&nbsp;:
                    <select name="action">
                      <option selected="selected" value="">---------</option>
                      <optgroup label="autres">
                        <option value="sell_cars">Sell the cars</option>
                      </optgroup>
                    </select>
                  </label>
                  <button value="0" name="index" title="Execute the selected action" class="button" type="submit">Execute</button>
                </div>
              </form>
              <div>
                <div><input type="checkbox" name="_selected_action" value="%(al1)d" />Alfa-Romeo - Giullia</div>
                <div><input type="checkbox" name="_selected_action" value="%(al2)d" />Alfa-Romeo - Sprint</div>
                <div><input type="checkbox" name="_selected_action" value="%(sa1)d" />Saab - 9.3</div>
                <div><input type="checkbox" name="_selected_action" value="%(sa2)d" />Saab - 9.5</div>
                <div><input type="checkbox" name="_selected_action" value="%(sa3)d" />Saab - 900</div>
                <div><input type="checkbox" name="_selected_action" value="%(su1)d" />Subaru - Forester</div>
                <div><input type="checkbox" name="_selected_action" value="%(su2)d" />Subaru - Impreza</div>
              <div>
            </html>""" % {
                'al1': self.c4.pk,
                'al2': self.c5.pk,
                'sa1': self.c1.pk,
                'sa2': self.c3.pk,
                'sa3': self.c2.pk,
                'su1': self.c6.pk,
                'su2': self.c7.pk,
            })

    def test_filtering_generic_list(self):
        request = self.rf.get('/cars?page=2&q=alfa')
        request.user = self.user
        search = [('Name', 'name__icontains'), ('Brand', 'brand__icontains')]
        response = generic.generic_list(request, self.queryset, self.columns, self.template,
                                        Car, ClassAdmin=CarAdmin, actions=self.actions, search=search)
        self.assertHTMLEqual(response.content.decode('utf8'), """
            <html>
              <form action="" method="post">
                <div class="actions">
                  <label>Action&nbsp;:
                    <select name="action">
                      <option selected="selected" value="">---------</option>
                      <optgroup label="autres">
                        <option value="sell_cars">Sell the cars</option>
                      </optgroup>
                    </select>
                  </label>
                  <button value="0" name="index" title="Execute the selected action" class="button" type="submit">Execute</button>
                </div>
              </form>
              <div>
                <div><input type="checkbox" name="_selected_action" value="%(al1)d" />Alfa-Romeo - Giullia</div>
                <div><input type="checkbox" name="_selected_action" value="%(al2)d" />Alfa-Romeo - Sprint</div>
              <div>
            </html>""" % {
                'al1': self.c4.pk,
                'al2': self.c5.pk,
            })

    def test_post_without_selected_ation_generic_list(self):
        request = self.rf.post('/cars?page=2', {})
        request.user = self.user
        request._messages = MessageStorage(request)
        response = generic.generic_list(request, self.queryset, self.columns, self.template,
                                        Car, ClassAdmin=CarAdmin, actions=self.actions)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, '/cars?page=2')
        self.assertEqual(list(Car.objects.filter(for_sale=True).order_by('name').values_list('pk', flat=True)),
                         [])

    def test_post_without_selected_items_generic_list(self):
        request = self.rf.post('/cars?page=2', {'action': 'sell_cars'})
        request.user = self.user
        request._messages = MessageStorage(request)
        response = generic.generic_list(request, self.queryset, self.columns, self.template,
                                        Car, ClassAdmin=CarAdmin, actions=self.actions)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, '/cars?page=2')
        self.assertEqual(list(Car.objects.filter(for_sale=True).order_by('name').values_list('pk', flat=True)),
                         [])

    def test_post_generic_list(self):
        request = self.rf.post('/cars?page=2', {'action': 'sell_cars',
                                                '_selected_action': [self.c1.pk, self.c4.pk]})
        request.user = self.user
        response = generic.generic_list(request, self.queryset, self.columns, self.template,
                                        Car, ClassAdmin=CarAdmin, actions=self.actions)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, '/cars?page=2')
        self.assertEqual(list(Car.objects.filter(for_sale=True).order_by('name').values_list('pk', flat=True)),
                         [self.c1.pk, self.c4.pk])


class GenericListFiltersTest(BaseGenericListTest):
    template = 'generic_list_filters_test.html'

    @classmethod
    def setUpTestData(cls):
        super(GenericListFiltersTest, cls).setUpTestData()
        cls.p1 = Person.objects.create(first_name='Stanislas', last_name='Guerra')
        cls.p2 = Person.objects.create(first_name='Michael', last_name='Schumacher')

    def test_boolean_filters_generic_list(self):
        request = self.rf.get('/cars?page=2')
        request.user = self.user
        filters = [('for_sale', )]
        response = generic.generic_list(request, self.queryset, self.columns, self.template,
                                        Car, ClassAdmin=CarAdmin, actions=self.actions, filters=filters)
        self.assertHTMLEqual(response.content.decode('utf8'), """
            <div>
              <h3><a></a><input type="hidden" name="filter_key" value="for sale" />By for sale</h3>
              <ul>
                <li class="selected"><a href="?">All</a></li>
                <li><a href="?for_sale__exact=1">Oui</a></li>
                <li><a href="?for_sale__exact=0">Non</a></li>
              </ul>
            </div>"""
        )

    def test_fk_filters_generic_list(self):
        request = self.rf.get('/cars?page=2')
        request.user = self.user
        filters = [('last_driver', {
            'choices': Person.objects.all().order_by('last_name'),
            'empty_choice': True,
        })]
        response = generic.generic_list(request, self.queryset, self.columns, self.template,
                                        Car, ClassAdmin=CarAdmin, actions=self.actions, filters=filters)
        self.assertHTMLEqual(response.content.decode('utf8'), """
            <div>
              <h3><a></a><input type="hidden" name="filter_key" value="last driver" />By last driver</h3>
              <ul>
                <li class="selected"><a href="?">All</a></li>
                <li><a href="?last_driver__exact=%d">Stanislas Guerra</a></li>
                <li><a href="?last_driver__exact=%d">Michael Schumacher</a></li>
                <li><a href="?last_driver__isnull=True">Aucun</a></li>
              </ul>
            </div>""" % (self.p1.pk, self.p2.pk)
        )

    def test_fk_filters_filtered_generic_list(self):
        """The url is filtered by one of the filter values."""

        request = self.rf.get('/cars?page=2&last_driver__exact=%d' % self.p2.pk)
        request.user = self.user
        filters = [('last_driver', {
            'choices': Person.objects.all().order_by('last_name'),
            'empty_choice': True,
        })]
        response = generic.generic_list(request, self.queryset, self.columns, self.template,
                                        Car, ClassAdmin=CarAdmin, actions=self.actions, filters=filters)
        self.assertHTMLEqual(response.content.decode('utf8'), """
            <div>
              <h3><a></a><input type="hidden" name="filter_key" value="last driver" />By last driver</h3>
              <ul>
                <li><a href="?">All</a></li>
                <li><a href="?last_driver__exact=%d">Stanislas Guerra</a></li>
                <li class="selected"><a href="?last_driver__exact=%d">Michael Schumacher</a></li>
                <li><a href="?last_driver__isnull=True">Aucun</a></li>
              </ul>
            </div>""" % (self.p1.pk, self.p2.pk)
        )



from django.db import models


class Car(models.Model):
    name = models.CharField(max_length=100)
    brand = models.CharField(max_length=100)
    description = models.TextField(null=True, blank=True)
    owner = models.ForeignKey('Person', on_delete=models.CASCADE, null=True)

    def __str__(self):
        return self.name


class Person(models.Model):
    name = models.CharField(max_length=100)
    lastname = models.CharField(max_length=100)
    gender = models.CharField(choices=[('male', 'Male'), ('female', 'Female')], max_length=6)



# -*- coding: utf-8 -*-

import unittest


class ActionsTest(unittest.TestCase):
    def test_regroup_actions(self):
        from django_stachoutils.views.actions import regroup_actions

        def action1():
            pass
        action1.group = "groupA"

        def action2():
            pass
        action2.group = "groupB"

        def action3():
            pass
        action3.group = "groupA"

        def action4():
            pass

        self.assertEqual(
            regroup_actions([action1, action2, action3]),
            [('groupA', [action1, action3]),
             ('groupB', [action2])]
        )

        self.assertEqual(
            regroup_actions([action1, action2, action3, action4]),
            [('groupA', [action1, action3]),
             ('groupB', [action2]),
             ('autres', [action4])]
        )


def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(ActionsTest)
    return suite



from django.db import models


class Car(models.Model):
    name = models.CharField(max_length=100)
    brand = models.CharField(max_length=100)
    type = models.CharField(max_length=100)
    horsepower = models.IntegerField()
    weight = models.IntegerField()
    top_speed = models.IntegerField()
    sales = models.IntegerField()

    def __str__(self):
        return self.name



from django.test import TestCase
from django import VERSION as DJ_VERSION
from django_stachoutils import models
from .models import Car, Person


class ModelsTest(TestCase):
    def test_get_obj_dict(self):
        person = Person.objects.create(name='Stan', lastname='Guerra', gender='male')
        car = Car.objects.create(brand='Saab', name='9.3', owner=person)
        expected = {
            'brand': 'Saab',
            'description': None,
            'name': '9.3',
            'owner_id': person.pk
        }
        if DJ_VERSION < (2, 0):
            expected['_owner_cache'] = person

        self.assertEqual(models.get_obj_dict(car), expected)



# -*- coding: utf-8 -*-

import logging
import logging.handlers
from django.core import mail
from django.conf import settings
from django.contrib import messages
from . import strip_accents

levelname_to_int = {
    'DEBUG': messages.DEBUG,
    'INFO': messages.INFO,
    'WARNING': messages.WARNING,
    'ERROR': messages.ERROR,
}


# Kept for backward compatibility.
class NullHandler(logging.NullHandler):
    pass


# http://www.red-dove.com/python_logging.html
class BufferingSMTPHandler(logging.handlers.BufferingHandler):
    def __init__(self, mailhost, fromaddr, toaddrs, subject, capacity, fmt="%(asctime)s %(levelname)-5s %(message)s"):
        logging.handlers.BufferingHandler.__init__(self, capacity)
        self.mailhost = mailhost
        self.mailport = None
        self.fromaddr = fromaddr
        self.toaddrs = toaddrs
        self.subject = subject
        self.setFormatter(logging.Formatter(fmt))

    def flush(self):
        if len(self.buffer) > 0:
            try:
                import smtplib
                port = self.mailport
                if not port:
                    port = smtplib.SMTP_PORT
                smtp = smtplib.SMTP(self.mailhost, port)
                msg = u"From: {}\r\nTo: {}\r\nSubject: {}\r\n\r\n".format(
                    self.fromaddr,
                    ",".join(self.toaddrs),
                    self.subject
                ).encode('utf8')
                for record in self.buffer:
                    msg_bit = self.format(record)
                    msg = msg + strip_accents(msg_bit).encode('ascii') + b"\r\n"
                smtp.sendmail(self.fromaddr, self.toaddrs, msg)
                smtp.quit()
            except:
                for record in self.buffer:
                    self.handleError(record)
            self.buffer = []


class BaseEmailHandler(logging.Handler):
    """Beneficie de la conf SMTP dans settings par rapport à logging.handlers.SMTPHandler. """
    def __init__(self, subject=""):
        """Allow one to set the subject in settings.LOGGING DictConfig. """
        logging.Handler.__init__(self)
        self.subject = subject

    def emit(self, record):
        subject = u'%s[%s] : %s' % (settings.EMAIL_SUBJECT_PREFIX, record.levelname, self.subject)
        message = record.msg
        mail.send_mail(subject, message, settings.DEFAULT_FROM_EMAIL, self.recipient_list,
                       fail_silently=True)


class AdminEmailHandler(BaseEmailHandler):
    def emit(self, record):
        self.recipient_list = [email for name, email in settings.ADMINS]
        BaseEmailHandler.emit(self, record)


class BufferingAdminEmailHandler(BufferingSMTPHandler):
    def __init__(self, subject="", capacity=1000):
        BufferingSMTPHandler.__init__(self, settings.EMAIL_HOST, settings.SERVER_EMAIL,
                                      [email for name, email in settings.ADMINS],
                                      subject, capacity)

    def flush(self):
        if len(self.buffer) > 0:
            msg = "From: %s\r\nTo: %s\r\nSubject: %s\r\n\r\n" % (
                self.fromaddr,
                ",".join(self.toaddrs),
                self.subject)
            body = "\r\n".join([self.format(r) for r in self.buffer])
            email = mail.EmailMessage(self.subject, msg + body, self.fromaddr, self.toaddrs)
            email.send()
            self.buffer = []


class MessagelHandler(logging.Handler):
    """An handler proxiing to messages. """

    def __init__(self):
        logging.Handler.__init__(self)

    def emit(self, record):
        messages.add_message(record.request, levelname_to_int[record.levelname], record.getMessage())



# -*- coding: utf-8 -*-

from past.builtins import basestring
from django.db.models import Aggregate, IntegerField

class SumExtra(Aggregate):
    function = 'SUM'
    template = "%(function)s(%(sum_expression)s)"


class SumCase(Aggregate):
    function = 'SUM'
    template = "%(function)s(CASE %(case_expression)s WHEN %(when)s THEN %(expressions)s ELSE 0 END)"

    def __init__(self, expression, **extra):
        if isinstance(extra['when'], basestring):
            extra['when'] = "'%s'" % extra['when']
        if extra['when'] is None:
            extra['when'] = True
        super(SumCase, self).__init__(
            expression,
            output_field=IntegerField(),
            **extra)


class CountCase(Aggregate):
    function = 'Count'
    template = "%(function)s(CASE %(expressions)s WHEN %(when)s THEN 1 ELSE null END)"

    def __init__(self, expression, **extra):
        if isinstance(extra['when'], basestring):
            quote = extra.get('quote', "'")
            extra['when'] = "(%s%s%s)" % (quote, extra['when'], quote)
        # Pas de cas d'utilisation. A vérifier car réécriture pour Django-1.8.
        if extra['when'] is None:
            extra['when'] = True
            extra['case'] += ' IS NULL '
        super(CountCase, self).__init__(
            expression,
            output_field=IntegerField(),
            **extra)


class CountCase2(Aggregate):
    function = 'Count'
    template = "%(function)s(CASE WHEN %(when)s THEN 1 ELSE null END)"

    def __init__(self, expression, **extra):
        if isinstance(extra['when'], basestring):
            quote = extra.get('quote', "'")
            extra['when'] = "(%s%s%s)" % (quote, extra['when'], quote)
        super(CountCase2, self).__init__(
            expression,
            output_field=IntegerField(),
            **extra)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-18 12:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('my_app', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='musicgenre',
            name='history',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-

import csv
import datetime
import decimal
import json
import zipfile

from io import BytesIO
from xml.dom import minidom
from django.conf import settings
from django.test import TestCase
from django_stachoutils import shortcuts
from pathlib import Path


class ResponseTest(TestCase):
    def test_macroman_response(self):
        data = u"B#Ω"
        response = shortcuts.macroman_text_response(data)
        self.assertEqual(response.content, b'B\xf0#\xbd')

    def test_macroman_response_with_filename(self):
        data = u"B#Ω"
        response = shortcuts.macroman_text_response(data, filename='export.txt')
        self.assertEqual(response.content, b'B\xf0#\xbd')
        self.assertEqual(response['Content-Disposition'], 'attachment; filename="export.txt"')

    def test_xml_response(self):
        data = u"""<?xml version="1.0"?><foo>This is a XML Document</foo>"""
        response = shortcuts.xml_response(data, filename="foo.xml")
        self.assertEqual(response.get('content-type'), 'application/xml')
        self.assertEqual(response.get('content-disposition'), 'attachment; filename="foo.xml"')

    def test_zip_response(self):
        files = [
            ("jardin.jpg", Path(f"{settings.MEDIA_ROOT}/jardin.jpg").read_bytes()),
            ("terrasse.jpg", Path(f"{settings.MEDIA_ROOT}/terrasse.jpg").read_bytes()),
        ]
        response = shortcuts.zip_response(files, filename='my_archive.zip')
        fp = BytesIO()
        fp.write(response.content)
        archive = zipfile.ZipFile(fp, mode='r')
        self.assertEqual(archive.namelist(), ['jardin.jpg', 'terrasse.jpg'])

    def test_csv_response(self):
        rows = [
            [u"Numéro de Serie", u"Marque", u"Modèle", u"Année"],
            [u"SB2323", u"Saab", u"9.3 2.0t", u"2007"],
            [u"LDNV005", u"Lada", u"Niva", u"2009"],
        ]
        response = shortcuts.csv_response(rows, "my_data.csv",
                                          {'delimiter': '|',
                                           'quoting': csv.QUOTE_MINIMAL})
        self.assertEqual(
            response.content,
            b'Num\xc3\xa9ro de Serie|Marque|Mod\xc3\xa8le|Ann\xc3\xa9e\r\nSB2323|Saab|9.3 2.0t|2007\r\nLDNV005|Lada|Niva|2009\r\n'
        )


class ResponseExtrasTest(TestCase):
    def test_encode_default(self):
        self.assertJSONEqual(json.dumps({"date": datetime.date(2018, 3, 14),
                                         "price": decimal.Decimal("14.99")},
                                        default=shortcuts.encode_default),
                             '{"date": "2018-03-14", "price": 14.99}')

    def test_encode_default_raises_exception(self):
        from .models import Car
        self.assertRaises(TypeError, json.dumps, {
            "date": datetime.date(2018, 3, 14),
            "price": decimal.Decimal("14.99"),
            "car": Car.objects.create(brand="Saab", name="9.3"),
        }, default=shortcuts.encode_default)

    def test_get_object_or_none(self):
        from .models import Car
        c1 = Car.objects.create(brand="Subaru", name="Impreza")
        Car.objects.create(brand="Saab", name="9.3")
        self.assertIsNone(shortcuts.get_object_or_none(Car, brand="Ford", name="Mustang"))
        self.assertEqual(shortcuts.get_object_or_none(Car, brand="Subaru", name="Impreza"), c1)

    def test_create_and_append_element(self):
        impl = minidom.getDOMImplementation('')
        doc = impl.createDocument(None, 'my_elements', None)
        node = shortcuts.createAndAppendElement(doc, doc.documentElement, 'my_node')
        self.assertXMLEqual(node.toxml(), '<my_node/>')

    def test_create_and_append_element_with_text(self):
        impl = minidom.getDOMImplementation('')
        doc = impl.createDocument(None, 'my_elements', None)
        node = shortcuts.createAndAppendElement(doc, doc.documentElement, 'my_node', 'Some text')
        self.assertXMLEqual(node.toxml(), '<my_node>Some text</my_node>')



# -*- coding: utf-8 -*-

from django.db import connection
from django.test import TestCase
from unittest import skipIf
from .models import Car


class AggregateTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.c1 = Car.objects.create(brand=u"Saab", name=u"9-3 2.0T", type=u"Coupé",
                                    horsepower=200, weight=1300, top_speed=230, sales=150000)
        cls.c2 = Car.objects.create(brand=u"Saab", name=u"900 Turbo16", type=u"Coupé",
                                    horsepower=175, weight=1200, top_speed=215, sales=225000)
        cls.c3 = Car.objects.create(brand=u"Alfa Roméo", name=u"Giulietta 2.0", type=u"Sedan",
                                    horsepower=130, weight=1100, top_speed=185, sales=115000)
        cls.c4 = Car.objects.create(brand=u"Alfa Roméo", name=u"GTV", type=u"Coupé",
                                    horsepower=150, weight=1050, top_speed=195, sales=65000)
        cls.c5 = Car.objects.create(brand=u"Lada", name=u"Niva", type=u"4x4",
                                    horsepower=70, weight=1100, top_speed=150, sales=300000)

    def test_count_case(self):
        from django_stachoutils.sql import CountCase
        qset = Car.objects.all().aggregate(nb_coupes=CountCase('type', when=u"Coupé"))
        self.assertEqual(qset['nb_coupes'], 3)

    def test_count_case2(self):
        from django_stachoutils.sql import CountCase2
        qset = Car.objects.all().aggregate(nb_rapides=CountCase2('type',
                                                                 when=u"top_speed > 190",
                                                                 quote=""))
        self.assertEqual(qset['nb_rapides'], 3)

    def test_sum_case(self):
        from django_stachoutils.sql import SumCase
        qset = Car.objects.all().aggregate(
            nb_ventes_coupes=SumCase('sales',
                                     case_expression="type",
                                     when=u"Coupé"))
        self.assertEqual(qset['nb_ventes_coupes'], 440000)

    @skipIf(connection.vendor == 'sqlite', "SQLite does not respect the cast of integer")
    def test_sum_extra(self):
        from django_stachoutils.sql import SumExtra
        qset = Car.objects.all().aggregate(
            total_horses_per_kg=SumExtra('horsepower',
                                         sum_expression="horsepower/(1.0*weight)"))
        self.assertEqual(qset['total_horses_per_kg'], (200+175+130+150+70)/(1.0*(1300+1200+1100+1050+1100)))



# -*- coding: utf-8 -*-

import hashlib
import time

INTEGRITY_CHECK_MAX_ATTEMPTS = 10


def simple_decorator(decorator):
    def new_decorator(f):
        g = decorator(f)
        g.__name__ = f.__name__
        g.__doc__ = f.__doc__
        g.__dict__.update(f.__dict__)
        return g
    new_decorator.__name__ = decorator.__name__
    new_decorator.__doc__ = decorator.__doc__
    new_decorator.__dict__.update(decorator.__dict__)
    return new_decorator


@simple_decorator
def check_file_integrity(view_func):
    def new_func(file_path, *args, **kwargs):
        delay, attempts = 0.7, 0
        digest1, digest2 = None, None
        logger = kwargs.get('logger', None)

        with open(file_path, 'rb') as f:
            digest2 = hexdigest_file(f)

        while digest1 != digest2:
            if logger and digest1:
                logger.warning(f"File integrity check on {file_path} failed. Waiting {delay} seconds.")
            if attempts >= INTEGRITY_CHECK_MAX_ATTEMPTS:
                if logger:
                    logger.error(f"File integrity check on {file_path} failed. Maximum attempts reached.")
                return
            digest1 = digest2
            time.sleep(delay)
            attempts += 1
            delay = delay * 2
            with open(file_path, 'rb') as f:
                digest2 = hexdigest_file(f)

            if logger:
                logger.debug(f"Integrity check on {file_path}. attempt {attempts}."
                             f" digest1: {digest1}, digest2: {digest2}")
        return view_func(file_path, *args, **kwargs)
    return new_func


def hexdigest_file(f, chunk_size=8192):
    md5 = hashlib.md5()
    for chunk in iter(lambda: f.read(chunk_size), b""):
        md5.update(chunk)
    f.seek(0)
    return md5.hexdigest()
